@@ -1,6 +1,6 @@
 // graph-tool -- a general graph modification and manipulation thingy
 //
-// Copyright (C) 2006-2023 Tiago de Paula Peixoto <tiago@skewed.de>
+// Copyright (C) 2006-2025 Tiago de Paula Peixoto <tiago@skewed.de>
 //
 // This program is free software; you can redistribute it and/or modify it under
 // the terms of the GNU Lesser General Public License as published by the Free
@@ -25,12 +25,12 @@
 using namespace std;
 using namespace graph_tool;
 
-size_t eigentrust(GraphInterface& g, boost::any c, boost::any t,
+size_t eigentrust(GraphInterface& g, std::any c, std::any t,
                   double epslon, size_t max_iter)
 {
-    if (!belongs<writable_edge_scalar_properties>()(c))
+    if (!belongs(writable_edge_scalar_properties, c))
         throw ValueException("edge property must be writable");
-    if (!belongs<vertex_floating_properties>()(t))
+    if (!belongs(vertex_floating_properties, t))
         throw ValueException("vertex property must be of floating point"
                              " value type");
 
@@ -44,7 +44,7 @@ size_t eigentrust(GraphInterface& g, boost::any c, boost::any t,
                   g.get_edge_index(), std::forward<decltype(a2)>(a2),
                   std::forward<decltype(a3)>(a3), epslon, max_iter, iter);
          },
-         writable_edge_scalar_properties(), vertex_floating_properties())(c, t);
+         writable_edge_scalar_properties, vertex_floating_properties)(c, t);
     return iter;
 }
 
