@@ -1,6 +1,6 @@
 // graph-tool -- a general graph modification and manipulation thingy
 //
-// Copyright (C) 2006-2023 Tiago de Paula Peixoto <tiago@skewed.de>
+// Copyright (C) 2006-2025 Tiago de Paula Peixoto <tiago@skewed.de>
 //
 // This program is free software; you can redistribute it and/or modify it under
 // the terms of the GNU Lesser General Public License as published by the Free
@@ -23,20 +23,20 @@
 using namespace std;
 using namespace graph_tool;
 
-long double eigenvector(GraphInterface& g, boost::any w, boost::any c,
+long double eigenvector(GraphInterface& g, std::any w, std::any c,
                         double epsilon, size_t max_iter)
 {
-    if (!w.empty() && !belongs<writable_edge_scalar_properties>()(w))
+    if (w.has_value() && !belongs(writable_edge_scalar_properties, w))
         throw ValueException("edge property must be writable");
-    if (!belongs<vertex_floating_properties>()(c))
+    if (!belongs(vertex_floating_properties, c))
         throw ValueException("vertex property must be of floating point"
                              " value type");
 
     typedef UnityPropertyMap<int,GraphInterface::edge_t> weight_map_t;
-    typedef boost::mpl::push_back<writable_edge_scalar_properties, weight_map_t>::type
-        weight_props_t;
+    auto weight_props = hana::append(writable_edge_scalar_properties,
+                                    hana::type<weight_map_t>());
 
-    if(w.empty())
+    if (!w.has_value())
         w = weight_map_t();
 
     long double eig = 0;
@@ -49,7 +49,7 @@ long double eigenvector(GraphInterface& g, boost::any w, boost::any c,
                   std::forward<decltype(a2)>(a2),
                   std::forward<decltype(a3)>(a3), epsilon, max_iter, eig);
          },
-         weight_props_t(), vertex_floating_properties())(w, c);
+         weight_props, vertex_floating_properties)(w, c);
     return eig;
 }
 
