@@ -1,6 +1,6 @@
 // graph-tool -- a general graph modification and manipulation thingy
 //
-// Copyright (C) 2006-2023 Tiago de Paula Peixoto <tiago@skewed.de>
+// Copyright (C) 2006-2025 Tiago de Paula Peixoto <tiago@skewed.de>
 //
 // This program is free software; you can redistribute it and/or modify it under
 // the terms of the GNU Lesser General Public License as published by the Free
@@ -81,16 +81,16 @@ struct deleted_key<std::string>
 
 pair<double,double>
 assortativity_coefficient(GraphInterface& gi, GraphInterface::deg_t deg,
-                          boost::any weight)
+                          std::any weight)
 {
     typedef UnityPropertyMap<size_t,GraphInterface::edge_t> weight_map_t;
-    typedef boost::mpl::push_back<edge_scalar_properties, weight_map_t>::type
-        weight_props_t;
+    auto weight_props = hana::append(edge_scalar_properties,
+                                     hana::type<weight_map_t>());
 
-    if (!weight.empty() && !belongs<edge_scalar_properties>()(weight))
+    if (weight.has_value() && !belongs(edge_scalar_properties, weight))
         throw ValueException("weight edge property must have a scalar value type");
 
-    if(weight.empty())
+    if(!weight.has_value())
         weight = weight_map_t();
 
     double a = 0, a_err = 0;
@@ -98,27 +98,27 @@ assortativity_coefficient(GraphInterface& gi, GraphInterface::deg_t deg,
         (gi,
          [&](auto&& graph, auto&& a2, auto&& a3)
          {
-             return get_assortativity_coefficient()
+             get_assortativity_coefficient()
                  (std::forward<decltype(graph)>(graph),
                   std::forward<decltype(a2)>(a2),
                   std::forward<decltype(a3)>(a3), a, a_err);
          },
-         all_selectors(), weight_props_t())(degree_selector(deg), weight);
+         all_selectors, weight_props)(degree_selector(deg), weight);
     return make_pair(a, a_err);
 }
 
 pair<double,double>
 scalar_assortativity_coefficient(GraphInterface& gi, GraphInterface::deg_t deg,
-                                 boost::any weight)
+                                 std::any weight)
 {
     typedef UnityPropertyMap<size_t,GraphInterface::edge_t> weight_map_t;
-    typedef boost::mpl::push_back<edge_scalar_properties, weight_map_t>::type
-        weight_props_t;
+    auto weight_props = hana::append(edge_scalar_properties,
+                                     hana::type<weight_map_t>());
 
-    if (!weight.empty() && !belongs<edge_scalar_properties>()(weight))
+    if (weight.has_value() && !belongs(edge_scalar_properties, weight))
         throw ValueException("weight edge property must have a scalar value type");
 
-    if(weight.empty())
+    if(!weight.has_value())
         weight = weight_map_t();
 
     double a = 0, a_err = 0;
@@ -126,12 +126,12 @@ scalar_assortativity_coefficient(GraphInterface& gi, GraphInterface::deg_t deg,
         (gi,
          [&](auto&& graph, auto&& a2, auto&& a3)
          {
-             return get_scalar_assortativity_coefficient()
+             get_scalar_assortativity_coefficient()
                  (std::forward<decltype(graph)>(graph),
                   std::forward<decltype(a2)>(a2),
                   std::forward<decltype(a3)>(a3), a, a_err);
          },
-         scalar_selectors(), weight_props_t())(degree_selector(deg), weight);
+         all_selectors, weight_props)(degree_selector(deg), weight);
     return make_pair(a, a_err);
 }
 
