@@ -1,6 +1,6 @@
 // graph-tool -- a general graph modification and manipulation thingy
 //
-// Copyright (C) 2006-2023 Tiago de Paula Peixoto <tiago@skewed.de>
+// Copyright (C) 2006-2025 Tiago de Paula Peixoto <tiago@skewed.de>
 //
 // This program is free software; you can redistribute it and/or modify it under
 // the terms of the GNU Lesser General Public License as published by the Free
@@ -46,12 +46,13 @@ get_vertex_combined_correlation_histogram(GraphInterface& gi,
     bins[0] = xbin;
     bins[1] = ybin;
 
-    run_action<>()(gi, get_correlation_histogram<GetCombinedPair>(hist, bins,
-                                                                  ret_bins),
-                   scalar_selectors(), scalar_selectors(),
-                   boost::mpl::vector<dummy_weight>())
+    run_action<>(false)
+        (gi, get_correlation_histogram<GetCombinedPair>(hist, bins,
+                                                        ret_bins),
+         all_selectors, all_selectors,
+         boost::hana::tuple_t<dummy_weight>)
         (any(degree_selector(deg1)), any(degree_selector(deg2)),
-         boost::any(dummy_weight()));
+         std::any(dummy_weight()));
 
     return boost::python::make_tuple(hist, ret_bins);
 }
