@@ -1,6 +1,6 @@
 // graph-tool -- a general graph modification and manipulation thingy
 //
-// Copyright (C) 2006-2023 Tiago de Paula Peixoto <tiago@skewed.de>
+// Copyright (C) 2006-2025 Tiago de Paula Peixoto <tiago@skewed.de>
 //
 // This program is free software; you can redistribute it and/or modify it under
 // the terms of the GNU Lesser General Public License as published by the Free
@@ -22,12 +22,12 @@
 
 #include "graph_selectors.hh"
 #include "graph_properties.hh"
+#include "graph_python_interface.hh"
 
 #include "graph_discrete.hh"
 
 #include "random.hh"
 
-#include <boost/python.hpp>
 
 using namespace std;
 using namespace boost;
@@ -58,6 +58,15 @@ public:
         std::shuffle(active.begin(), active.end(), rng);
     }
 
+    void set_active(boost::python::object oa, rng_t& rng)
+    {
+        auto a = get_array<int64_t, 1>(oa);
+        auto& active = *_state._active;
+        active.clear();
+        active.insert(active.end(), a.begin(), a.end());
+        std::shuffle(active.begin(), active.end(), rng);
+    }
+
     python::object get_active()
     {
         return wrap_vector_not_owned(*_state._active);
@@ -65,11 +74,13 @@ public:
 
     size_t iterate_sync(size_t niter, rng_t& rng)
     {
+        GILRelease gil;
         return discrete_iter_sync(_g, _state, niter, rng);
     }
 
     size_t iterate_async(size_t niter, rng_t& rng)
     {
+        GILRelease gil;
         return discrete_iter_async(_g, _state, niter, rng);
     }
 
@@ -80,6 +91,7 @@ public:
              python::init<Graph&, smap_t, smap_t, python::dict, rng_t&>())
             .def("reset_active", &WrappedState<Graph,State>::reset_active)
             .def("get_active", &WrappedState<Graph,State>::get_active)
+            .def("set_active", &WrappedState<Graph,State>::set_active)
             .def("iterate_sync", &WrappedState<Graph,State>::iterate_sync)
             .def("iterate_async", &WrappedState<Graph,State>::iterate_async);
     }
@@ -90,15 +102,15 @@ private:
 };
 
 template <class State>
-python::object make_state(GraphInterface& gi, boost::any as, boost::any as_temp,
+python::object make_state(GraphInterface& gi, std::any as, std::any as_temp,
                           python::dict params, rng_t& rng)
 {
     typedef typename State::smap_t::checked_t smap_t;
-    smap_t s = boost::any_cast<smap_t>(as);
-    smap_t s_temp = boost::any_cast<smap_t>(as_temp);
+    smap_t s = std::any_cast<smap_t>(as);
+    smap_t s_temp = std::any_cast<smap_t>(as_temp);
 
     python::object state;
-    run_action<>()
+    run_action<>(false)
         (gi,
          [&](auto& g)
          {
@@ -112,24 +124,14 @@ python::object make_state(GraphInterface& gi, boost::any as, boost::any as_temp,
     return state;
 }
 
-struct add_ptr
-{
-    template <class T>
-    struct apply
-    {
-        typedef typename std::add_pointer<T>::type type;
-    };
-};
-
 template <class State>
 static void export_state()
 {
-    mpl::for_each<all_graph_views, add_ptr>
-        ([](auto g)
-         {
-             typedef typename std::remove_pointer<decltype(g)>::type g_t;
-             WrappedState<g_t,State>::python_export();
-         });
+    hana::for_each(all_graph_views,
+                   [](auto g)
+                   {
+                       WrappedState<typename decltype(g)::type,State>::python_export();
+                   });
 }
 
 template <template <bool...> class Base, bool... As>
@@ -141,8 +143,8 @@ static void export_SI_state()
 }
 
 template <template <bool...> class Base, bool... As>
-python::object make_SI_state(GraphInterface& gi, boost::any as,
-                             boost::any as_temp, python::dict params,
+python::object make_SI_state(GraphInterface& gi, std::any as,
+                             std::any as_temp, python::dict params,
                              rng_t& rng, bool weighted, bool constant_beta)
 {
     if (weighted)
@@ -217,6 +219,15 @@ REGISTER_MOD
      export_state<boolean_state>();
      def("make_boolean_state", &make_state<boolean_state>);
 
+     export_state<generalized_binary_state>();
+     def("make_generalized_binary_state", &make_state<generalized_binary_state>);
+
      export_state<kirman_state>();
      def("make_kirman_state", &make_state<kirman_state>);
- });
+
+     export_state<normal_state>();
+     def("make_normal_state", &make_state<normal_state>);
+
+     export_state<linear_normal_state>();
+     def("make_linear_normal_state", &make_state<linear_normal_state>);
+});
