@@ -1,6 +1,6 @@
 // graph-tool -- a general graph modification and manipulation thingy
 //
-// Copyright (C) 2006-2023 Tiago de Paula Peixoto <tiago@skewed.de>
+// Copyright (C) 2006-2025 Tiago de Paula Peixoto <tiago@skewed.de>
 //
 // This program is free software; you can redistribute it and/or modify it under
 // the terms of the GNU Lesser General Public License as published by the Free
@@ -65,9 +65,9 @@ struct get_edmonds_karp_max_flow
 
 
 void edmonds_karp_max_flow_dispatch(GraphInterface& gi, size_t src, size_t sink,
-                                    boost::any capacity, boost::any res)
+                                    std::any capacity, std::any res)
 {
-    run_action<graph_tool::detail::always_directed>()
+    run_action<decltype(graph_tool::always_directed)>()
         (gi,
          [&](auto&& graph, auto&& a2, auto&& a3)
          {
@@ -77,8 +77,8 @@ void edmonds_karp_max_flow_dispatch(GraphInterface& gi, size_t src, size_t sink,
                   std::forward<decltype(a2)>(a2),
                   std::forward<decltype(a3)>(a3));
          },
-         writable_edge_scalar_properties(),
-         writable_edge_scalar_properties())(capacity, res);
+         writable_edge_scalar_properties,
+         writable_edge_scalar_properties)(capacity, res);
 }
 
 #include <boost/python.hpp>
