@@ -1,6 +1,6 @@
 // graph-tool -- a general graph modification and manipulation thingy
 //
-// Copyright (C) 2006-2022 Tiago de Paula Peixoto <tiago@skewed.de>
+// Copyright (C) 2006-2025 Tiago de Paula Peixoto <tiago@skewed.de>
 //
 // This program is free software; you can redistribute it and/or modify it under
 // the terms of the GNU Lesser General Public License as published by the Free
@@ -24,15 +24,24 @@ using namespace std;
 using namespace boost;
 using namespace graph_tool;
 
-
 void price(GraphInterface& gi, size_t N, double gamma, double c, size_t m,
            rng_t& rng)
 {
     run_action<>()
         (gi,
-         [&](auto&& graph)
+         [&](auto& g)
          {
-             return get_price()
-                 (std::forward<decltype(graph)>(graph), N, gamma, c, m, rng);
+             return get_price(g, N, gamma, c, m, rng);
          })();
 }
+
+#include <boost/python.hpp>
+using namespace boost::python;
+
+#define __MOD__ generation
+#include "module_registry.hh"
+REGISTER_MOD
+([]
+ {
+     def("price", &price);
+ });
