@@ -1,6 +1,6 @@
 // graph-tool -- a general graph modification and manipulation thingy
 //
-// Copyright (C) 2006-2023 Tiago de Paula Peixoto <tiago@skewed.de>
+// Copyright (C) 2006-2025 Tiago de Paula Peixoto <tiago@skewed.de>
 //
 // This program is free software; you can redistribute it and/or modify it under
 // the terms of the GNU Lesser General Public License as published by the Free
@@ -134,20 +134,20 @@ size_t random_rewire(GraphInterface& gi, string strat, size_t niter,
                      bool no_sweep, bool self_loops, bool parallel_edges,
                      bool configuration, bool traditional, bool micro,
                      bool persist, boost::python::object corr_prob,
-                     boost::any apin, boost::any block, bool cache, rng_t& rng,
+                     std::any apin, std::any block, bool cache, rng_t& rng,
                      bool verbose)
 {
     PythonFuncWrap corr(corr_prob);
     size_t pcount = 0;
 
-    typedef eprop_map_t<uint8_t>::type emap_t;
+    typedef eprop_map_t<uint8_t> emap_t;
 
     emap_t::unchecked_t pin =
-        any_cast<emap_t>(apin).get_unchecked(gi.get_edge_index_range());
+        std::any_cast<emap_t>(apin).get_unchecked(gi.get_edge_index_range());
 
     if (strat == "erdos")
     {
-        run_action<graph_tool::detail::never_reversed>()
+        run_action<decltype(never_reversed)>(false)
             (gi,
              [&](auto&& graph)
              {
@@ -160,7 +160,7 @@ size_t random_rewire(GraphInterface& gi, string strat, size_t niter,
     }
     else if (strat == "configuration")
     {
-        run_action<graph_tool::detail::never_reversed>()
+        run_action<decltype(never_reversed)>(false)
             (gi,
              [&](auto&& graph)
              {
@@ -173,9 +173,9 @@ size_t random_rewire(GraphInterface& gi, string strat, size_t niter,
     }
     else if (strat == "constrained-configuration")
     {
-        if (block.empty())
+        if (!block.has_value())
         {
-            run_action<graph_tool::detail::never_reversed>()
+            run_action<decltype(never_reversed)>(false)
                 (gi,
                  [&](auto&& graph)
                  {
@@ -189,7 +189,7 @@ size_t random_rewire(GraphInterface& gi, string strat, size_t niter,
         }
         else
         {
-            run_action<graph_tool::detail::never_reversed>()
+            run_action<decltype(never_reversed)>(false)
                 (gi,
                  [&](auto&& graph, auto&& a2)
                  {
@@ -201,12 +201,12 @@ size_t random_rewire(GraphInterface& gi, string strat, size_t niter,
                           std::make_tuple(persist, cache, verbose), pcount, rng,
                           std::forward<decltype(a2)>(a2));
                  },
-                 vertex_properties())(block);
+                 vertex_properties)(block);
         }
     }
     else if (strat == "probabilistic-configuration")
     {
-        run_action<>()
+        run_action<>(false)
             (gi,
              [&](auto&& graph)
              {
@@ -219,7 +219,7 @@ size_t random_rewire(GraphInterface& gi, string strat, size_t niter,
     }
     else if (strat == "blockmodel")
     {
-        run_action<>()
+        run_action<>(false)
             (gi,
              [&](auto&& graph, auto&& a2)
              {
@@ -230,7 +230,7 @@ size_t random_rewire(GraphInterface& gi, string strat, size_t niter,
                      make_pair(niter, no_sweep),
                      std::make_tuple(persist, cache, verbose), pcount, rng);
              },
-             vertex_properties())(block);
+             vertex_properties)(block);
     }
     else
     {
