@@ -1,6 +1,6 @@
 // graph-tool -- a general graph modification and manipulation thingy
 //
-// Copyright (C) 2006-2023 Tiago de Paula Peixoto <tiago@skewed.de>
+// Copyright (C) 2006-2025 Tiago de Paula Peixoto <tiago@skewed.de>
 //
 // This program is free software; you can redistribute it and/or modify it under
 // the terms of the GNU Lesser General Public License as published by the Free
@@ -85,13 +85,12 @@ using namespace boost;
 using namespace graph_tool;
 
 void triangulation(GraphInterface& gi, boost::python::object points,
-                   boost::any pos, string type, bool periodic)
+                   std::any pos, string type, bool periodic)
 {
-    undirected_adaptor<GraphInterface::multigraph_t> g(gi.get_graph());
+    auto& g = get_undirected_adaptor(gi.get_graph());
     multi_array_ref<double,2> points_array = get_array<double,2>(points);
-    typedef property_map_type::apply
-        <vector<double>, GraphInterface::vertex_index_map_t>::type pos_type_t;
-    pos_type_t pos_map = any_cast<pos_type_t>(pos);
+    typedef vprop_map_t<vector<double>> pos_type_t;
+    pos_type_t pos_map = std::any_cast<pos_type_t>(pos);
 
     if (type == "simple")
     {
