@@ -1,6 +1,6 @@
 // graph-tool -- a general graph modification and manipulation thingy
 //
-// Copyright (C) 2006-2023 Tiago de Paula Peixoto <tiago@skewed.de>
+// Copyright (C) 2006-2025 Tiago de Paula Peixoto <tiago@skewed.de>
 //
 // This program is free software; you can redistribute it and/or modify it under
 // the terms of the GNU Lesser General Public License as published by the Free
@@ -32,7 +32,7 @@ template <class GraphSrc, class GraphTgt, class IndexMap, class SrcIndexMap,
 struct copy_vertex_property_dispatch
 {
     copy_vertex_property_dispatch(const GraphSrc& src, const GraphTgt& tgt,
-                                  boost::any& prop_src, boost::any& prop_tgt,
+                                  std::any& prop_src, std::any& prop_tgt,
                                   IndexMap& index_map,
                                   SrcIndexMap& src_vertex_index,
                                   TgtIndexMap& tgt_vertex_index,
@@ -45,8 +45,8 @@ struct copy_vertex_property_dispatch
 
     const GraphSrc& src;
     const GraphTgt& tgt;
-    boost::any& prop_src;
-    boost::any& prop_tgt;
+    std::any& prop_src;
+    std::any& prop_tgt;
     IndexMap& index_map;
     SrcIndexMap& src_vertex_index;
     TgtIndexMap& tgt_vertex_index;
@@ -55,12 +55,12 @@ struct copy_vertex_property_dispatch
     template <class PropertyMap>
     void operator()(PropertyMap) const
     {
-        PropertyMap* psrc = any_cast<PropertyMap>(&prop_src);
+        PropertyMap* psrc = std::any_cast<PropertyMap>(&prop_src);
         if (psrc == NULL)
             return;
-        if (prop_tgt.empty())
+        if (!prop_tgt.has_value())
             prop_tgt = PropertyMap(tgt_vertex_index);
-        PropertyMap* ptgt = any_cast<PropertyMap>(&prop_tgt);
+        PropertyMap* ptgt = std::any_cast<PropertyMap>(&prop_tgt);
         if (ptgt == NULL)
             return;
         found = true;
@@ -80,17 +80,23 @@ struct copy_vertex_property_dispatch
 
 template <class PropertyMaps, class GraphSrc, class GraphTgt,
           class IndexMap, class SrcIndexMap, class TgtIndexMap>
-void copy_vertex_property(boost::any& prop_src, boost::any& prop_tgt,
+void copy_vertex_property(PropertyMaps pmaps, std::any& prop_src, std::any& prop_tgt,
                           const GraphSrc& src, const GraphTgt& tgt,
                           IndexMap& index_map, SrcIndexMap& src_vertex_index,
                           TgtIndexMap& tgt_vertex_index)
 {
     bool found = false;
-    boost::mpl::for_each<PropertyMaps>(copy_vertex_property_dispatch<GraphSrc, GraphTgt,
-                                                                     IndexMap, SrcIndexMap,
-                                                                     TgtIndexMap>
-            (src, tgt, prop_src, prop_tgt, index_map, src_vertex_index,
-             tgt_vertex_index, found));
+    copy_vertex_property_dispatch<GraphSrc, GraphTgt,
+                                  IndexMap, SrcIndexMap,
+                                  TgtIndexMap>
+        dispatch(src, tgt, prop_src, prop_tgt, index_map, src_vertex_index,
+                 tgt_vertex_index, found);
+
+    hana::for_each(pmaps,
+                   [&](auto t)
+                   {
+                       dispatch(typename decltype(+t)::type());
+                   });
     if (!found)
         throw ValueException("Cannot find property map type.");
 }
@@ -100,7 +106,7 @@ template <class GraphSrc, class GraphTgt, class IndexMap, class SrcIndexMap>
 struct copy_edge_property_dispatch
 {
     copy_edge_property_dispatch(const GraphSrc& src, const GraphTgt& tgt,
-                                boost::any& prop_src, boost::any& prop_tgt,
+                                std::any& prop_src, std::any& prop_tgt,
                                 IndexMap& index_map,
                                 SrcIndexMap& src_edge_index,
                                 size_t max_src_edge_index,
@@ -113,8 +119,8 @@ struct copy_edge_property_dispatch
 
     const GraphSrc& src;
     const GraphTgt& tgt;
-    boost::any& prop_src;
-    boost::any& prop_tgt;
+    std::any& prop_src;
+    std::any& prop_tgt;
     IndexMap& index_map;
     SrcIndexMap& src_edge_index;
     size_t max_src_edge_index;
@@ -123,12 +129,12 @@ struct copy_edge_property_dispatch
     template <class PropertyMap>
     void operator()(PropertyMap) const
     {
-        PropertyMap* psrc = any_cast<PropertyMap>(&prop_src);
+        PropertyMap* psrc = std::any_cast<PropertyMap>(&prop_src);
         if (psrc == NULL)
             return;
-        if (prop_tgt.empty())
+        if (!prop_tgt.has_value())
             prop_tgt = PropertyMap(get(edge_index_t(), tgt));
-        PropertyMap* ptgt = any_cast<PropertyMap>(&prop_tgt);
+        PropertyMap* ptgt = std::any_cast<PropertyMap>(&prop_tgt);
         if (ptgt == NULL)
             return;
         found = true;
@@ -156,16 +162,22 @@ struct copy_edge_property_dispatch
 
 template <class PropertyMaps, class GraphSrc, class GraphTgt,
           class IndexMap, class SrcIndexMap>
-void copy_edge_property(boost::any& prop_src, boost::any& prop_tgt,
+void copy_edge_property(PropertyMaps pmaps, std::any& prop_src, std::any& prop_tgt,
                         const GraphSrc& src, const GraphTgt& tgt,
                         IndexMap& index_map, SrcIndexMap& src_vertex_index,
                         size_t max_src_edge_index)
 {
     bool found = false;
-    boost::mpl::for_each<PropertyMaps>(copy_edge_property_dispatch<GraphSrc, GraphTgt,
-                                                                   IndexMap, SrcIndexMap>
-            (src, tgt, prop_src, prop_tgt, index_map, src_vertex_index,
-             max_src_edge_index, found));
+    copy_edge_property_dispatch<GraphSrc, GraphTgt,
+                                IndexMap, SrcIndexMap>
+        dispatch (src, tgt, prop_src, prop_tgt, index_map, src_vertex_index,
+                  max_src_edge_index, found);
+
+    boost::hana::for_each(pmaps,
+                          [&](auto pmap)
+                          {
+                              dispatch(typename decltype(pmap)::type());
+                          });
     if (!found)
         throw ValueException("Cannot find property map type.");
 }
@@ -186,8 +198,8 @@ struct do_graph_copy
                     TgtEdgeIndexMap,
                     SrcEdgeIndexMap src_edge_index,
                     OrderMap vertex_order,
-                    vector<pair<std::reference_wrapper<boost::any>,std::reference_wrapper<boost::any>>>& vprops,
-                    vector<pair<std::reference_wrapper<boost::any>,std::reference_wrapper<boost::any>>>& eprops) const
+                    vector<pair<std::reference_wrapper<std::any>,std::reference_wrapper<std::any>>>& vprops,
+                    vector<pair<std::reference_wrapper<std::any>,std::reference_wrapper<std::any>>>& eprops) const
     {
         vector<size_t> vs;
         for (auto v : vertices_range(src))
@@ -213,9 +225,9 @@ struct do_graph_copy
         }
 
         for (size_t i = 0; i < vprops.size(); ++i)
-            copy_vertex_property<writable_vertex_properties>
-                    (vprops[i].first.get(), vprops[i].second.get(),
-                     src, tgt, index_map, src_vertex_index, tgt_vertex_index);
+            copy_vertex_property(writable_vertex_properties,
+                                 vprops[i].first.get(), vprops[i].second.get(),
+                                 src, tgt, index_map, src_vertex_index, tgt_vertex_index);
 
 
         vector<typename graph_traits<GraphTgt>::edge_descriptor> edge_map(num_edges(src));
@@ -233,9 +245,9 @@ struct do_graph_copy
         }
 
         for (size_t i = 0; i < eprops.size(); ++i)
-            copy_edge_property<writable_edge_properties>
-                (eprops[i].first.get(), eprops[i].second.get(),
-                 src, tgt, edge_map, src_edge_index, max_src_edge_index);
+            copy_edge_property(writable_edge_properties,
+                               eprops[i].first.get(), eprops[i].second.get(),
+                               src, tgt, edge_map, src_edge_index, max_src_edge_index);
 
     }
 };
@@ -250,10 +262,8 @@ GraphInterface::GraphInterface(const GraphInterface& gi, bool keep_ref,
      _reversed(gi._reversed),
      _directed(gi._directed),
      _vertex_filter_map(_vertex_index),
-     _vertex_filter_invert(false),
      _vertex_filter_active(false),
      _edge_filter_map(_edge_index),
-     _edge_filter_invert(false),
      _edge_filter_active(false)
 {
     if (keep_ref)
@@ -266,20 +276,20 @@ GraphInterface::GraphInterface(const GraphInterface& gi, bool keep_ref,
         return;
     }
 
-    vector<pair<std::reference_wrapper<boost::any>,std::reference_wrapper<boost::any>>> vprops;
+    vector<pair<std::reference_wrapper<std::any>,std::reference_wrapper<std::any>>> vprops;
     for (int i = 0; i < python::len(ovprops); ++i)
     {
-        vprops.push_back(make_pair(std::ref(python::extract<boost::any&>(ovprops[i][0])()),
-                                   std::ref(python::extract<boost::any&>(ovprops[i][1])())));
+        vprops.push_back(make_pair(std::ref(python::extract<std::any&>(ovprops[i][0])()),
+                                   std::ref(python::extract<std::any&>(ovprops[i][1])())));
     }
-    vector<pair<std::reference_wrapper<boost::any>,std::reference_wrapper<boost::any>>> eprops;
+    vector<pair<std::reference_wrapper<std::any>,std::reference_wrapper<std::any>>> eprops;
     for (int i = 0; i < python::len(oeprops); ++i)
     {
-        eprops.push_back(make_pair(std::ref(python::extract<boost::any&>(oeprops[i][0])()),
-                                   std::ref(python::extract<boost::any&>(oeprops[i][1])())));
+        eprops.push_back(make_pair(std::ref(python::extract<std::any&>(oeprops[i][0])()),
+                                   std::ref(python::extract<std::any&>(oeprops[i][1])())));
     }
 
-    boost::any avorder = python::extract<boost::any>(vorder)();
+    std::any avorder = python::extract<std::any>(vorder)();
     run_action<>()
         (const_cast<GraphInterface&>(gi),
          [&](auto&& graph, auto&& a2)
@@ -289,6 +299,6 @@ GraphInterface::GraphInterface(const GraphInterface& gi, bool keep_ref,
                  _vertex_index, gi._edge_index, _edge_index,
                  std::forward<decltype(a2)>(a2), vprops, eprops);
          },
-         vertex_scalar_properties())(avorder);
+         vertex_scalar_properties)(avorder);
     // filters will be copied in python
 }
