@@ -1,6 +1,6 @@
 // graph-tool -- a general graph modification and manipulation thingy
 //
-// Copyright (C) 2006-2023 Tiago de Paula Peixoto <tiago@skewed.de>
+// Copyright (C) 2006-2025 Tiago de Paula Peixoto <tiago@skewed.de>
 //
 // This program is free software; you can redistribute it and/or modify it under
 // the terms of the GNU Lesser General Public License as published by the Free
@@ -133,57 +133,43 @@ private:
 // Property Maps
 // =============
 
-struct get_python_property
-{
-    template <class ValueType, class IndexMap>
-    void operator()(ValueType, IndexMap, dynamic_property_map& map,
-                    boost::python::object& pmap) const
-    {
-        typedef typename property_map_type::apply<ValueType, IndexMap>::type
-            map_t;
-        try
-        {
-            pmap = boost::python::object
-                (PythonPropertyMap<map_t>
-                 (dynamic_cast
-                  <boost::detail::dynamic_property_map_adaptor<map_t>&>(map)
-                  .base()));
-        } catch (bad_cast&) {}
-    }
-
-    template <class ValueType, class IndexMap>
-    void operator()(ValueType, IndexMap, boost::any& map,
-                    boost::python::object& pmap) const
-    {
-        typedef typename property_map_type::apply<ValueType, IndexMap>::type
-            map_t;
-        try
-        {
-            PythonPropertyMap<map_t> opmap(any_cast<map_t>(map));
-            pmap = boost::python::object(opmap);
-        }
-        catch (bad_any_cast&) {}
-    }
-
-};
-
 template <class IndexMap>
 boost::python::object find_property_map(dynamic_property_map& map, IndexMap)
 {
     boost::python::object pmap;
-    boost::mpl::for_each<value_types>(std::bind(get_python_property(),
-                                                std::placeholders::_1, IndexMap(), std::ref(map),
-                                                std::ref(pmap)));
+    hana::for_each(value_types,
+                   [&](auto t)
+                   {
+                       typedef typename decltype(+t)::type val_t;
+                       typedef property_map_t<val_t, IndexMap> map_t;
+                       try
+                       {
+                           pmap = boost::python::object
+                               (PythonPropertyMap<map_t>
+                                (dynamic_cast
+                                 <boost::detail::dynamic_property_map_adaptor<map_t>&>(map)
+                                 .base()));
+                       } catch (bad_cast&) {}
+                   });
     return pmap;
 }
 
 template <class IndexMap>
-boost::python::object find_property_map(boost::any& map, IndexMap)
+boost::python::object find_property_map(std::any& map, IndexMap)
 {
     boost::python::object pmap;
-    boost::mpl::for_each<value_types>(std::bind(get_python_property(),
-                                                std::placeholders::_1, IndexMap(), std::ref(map),
-                                                std::ref(pmap)));
+    hana::for_each(value_types,
+                   [&](auto t)
+                   {
+                       typedef typename decltype(+t)::type val_t;
+                       typedef property_map_t<val_t, IndexMap> map_t;
+                       try
+                       {
+                           PythonPropertyMap<map_t> opmap(std::any_cast<map_t>(map));
+                           pmap = boost::python::object(opmap);
+                       }
+                       catch (std::bad_any_cast&) {}
+                   });
     return pmap;
 }
 
@@ -203,14 +189,13 @@ struct check_value_type
     {
         try
         {
-            typedef typename property_map_type::apply<ValueType, IndexMap>::type
-                map_t;
+            typedef property_map_t<ValueType, IndexMap> map_t;
             map_t vector_map(_index_map);
-            vector_map[_key] = any_cast<ValueType>(_value);
+            vector_map[_key] = boost::any_cast<ValueType>(_value);
             _map = new boost::detail::dynamic_property_map_adaptor<map_t>
                 (vector_map);
         }
-        catch (bad_any_cast&) {}
+        catch (boost::bad_any_cast&) {}
     }
     IndexMap _index_map;
     const key_t& _key;
@@ -233,31 +218,43 @@ struct create_dynamic_map
                                                   const boost::any& key,
                                                   const boost::any& value)
     {
-        dynamic_property_map* map;
+        dynamic_property_map* map = nullptr;
         try
         {
-            boost::mpl::for_each<value_types>
-                (check_value_type<VertexIndexMap>(_vertex_map,
-                                                  any_cast<vertex_t>(key),
-                                                  value, map));
+            hana::for_each(value_types,
+                           [&](auto t)
+                           {
+                               typedef typename decltype(+t)::type val_t;
+                               check_value_type<VertexIndexMap>(_vertex_map,
+                                                                boost::any_cast<vertex_t>(key),
+                                                                value, map)(val_t());
+                           });
         }
-        catch (bad_any_cast&)
+        catch (boost::bad_any_cast&)
         {
             try
             {
-                boost::mpl::for_each<value_types>
-                    (check_value_type<EdgeIndexMap>(_edge_map,
-                                                    any_cast<edge_t>(key),
-                                                    value, map));
+                hana::for_each(value_types,
+                               [&](auto t)
+                               {
+                                   typedef typename decltype(+t)::type val_t;
+                                   check_value_type<EdgeIndexMap>(_edge_map,
+                                                                  boost::any_cast<edge_t>(key),
+                                                                  value, map)(val_t());
+                               });
             }
-            catch (bad_any_cast&)
+            catch (boost::bad_any_cast&)
             {
                 ConstantPropertyMap<size_t,graph_property_tag> graph_index(0);
-                boost::mpl::for_each<value_types>
-                    (check_value_type<ConstantPropertyMap<size_t,
-                                                          graph_property_tag> >
-                     (graph_index, any_cast<graph_property_tag>(key),
-                      value, map));
+                hana::for_each(value_types,
+                               [&](auto t)
+                               {
+                                   typedef typename decltype(+t)::type val_t;
+                                   check_value_type<ConstantPropertyMap<size_t,
+                                                                        graph_property_tag> >
+                                       (graph_index, boost::any_cast<graph_property_tag>(key),
+                                        value, map)(val_t());
+                               });
             }
         }
         return DP_SMART_PTR<dynamic_property_map>(map);
@@ -277,7 +274,9 @@ void build_stream(boost::iostreams::filtering_stream<boost::iostreams::input>& s
 {
     stream.reset();
     if (file == "-")
+    {
         stream.push(std::cin);
+    }
     else
     {
         if (pfile == boost::python::object())
@@ -344,7 +343,7 @@ boost::python::tuple GraphInterface::read_from_file(string file,
         boost::python::dict vprops, eprops, gprops;
         if (format == "gt")
         {
-            vector<pair<string, boost::any>> agprops, avprops, aeprops;
+            vector<pair<string, std::any>> agprops, avprops, aeprops;
             stream.exceptions(ios_base::badbit | ios_base::failbit | ios_base::eofbit);
             _directed = read_graph(stream, *_mg, agprops, avprops, aeprops, igp,
                                    ivp, iep);
@@ -432,9 +431,9 @@ struct do_write_to_binary_file
 {
     template <class Graph, class IndexMap>
     void operator()(ostream& stream, Graph& g, IndexMap index_map, size_t N,
-                    bool directed, vector<pair<string, boost::any >> & gprops,
-                    vector<pair<string, boost::any >> & vprops,
-                    vector<pair<string, boost::any >> & eprops) const
+                    bool directed, vector<pair<string, std::any >> & gprops,
+                    vector<pair<string, std::any >> & vprops,
+                    vector<pair<string, std::any >> & eprops) const
     {
         write_graph(g, index_map, N, directed, gprops, vprops, eprops, stream);
     }
@@ -487,20 +486,19 @@ void GraphInterface::write_to_file(string file, boost::python::object pfile,
 
         if (format == "gt")
         {
-            typedef property_map_types::apply<value_types,
-                                              GraphInterface::graph_index_map_t>::type
-                graph_properties;
+            auto graph_properties =
+                get_prop_types<GraphInterface::graph_index_map_t>(value_types);
 
-            vector<pair<string, boost::any>> agprops, avprops, aeprops;
+            vector<pair<string, std::any>> agprops, avprops, aeprops;
             for (int i = 0; i < python::len(props); ++i)
             {
                 string name = python::extract<string>(props[i][0])();
-                boost::any p = python::extract<boost::any>(props[i][1].attr("get_map")())();
-                if (belongs<graph_properties>()(p))
+                std::any p = python::extract<std::any>(props[i][1].attr("get_map")())();
+                if (belongs(graph_properties, p))
                     agprops.push_back(make_pair(name, p));
-                if (belongs<vertex_properties>()(p))
+                if (belongs(vertex_properties, p))
                     avprops.push_back(make_pair(name, p));
-                if (belongs<edge_properties>()(p))
+                if (belongs(edge_properties, p))
                     aeprops.push_back(make_pair(name, p));
             }
 
@@ -518,7 +516,7 @@ void GraphInterface::write_to_file(string file, boost::python::object pfile,
                          return generate_index()
                              (std::forward<decltype(graph)>(graph), index_map);
                      })();
-                run_action<>()
+                run_action<>(false)
                     (*this,
                      [&](auto&& graph)
                      {
@@ -530,7 +528,7 @@ void GraphInterface::write_to_file(string file, boost::python::object pfile,
             }
             else
             {
-                run_action<>()
+                run_action<>(false)
                     (*this,
                      [&](auto&& graph)
                      {
@@ -549,8 +547,8 @@ void GraphInterface::write_to_file(string file, boost::python::object pfile,
             for (int i = 0; i < len(props); ++i)
             {
                 dynamic_property_map* pmap =
-                    any_cast<dynamic_property_map*>
-                    (boost::python::extract<boost::any>
+                    std::any_cast<dynamic_property_map*>
+                    (boost::python::extract<std::any>
                      (props[i][1].attr("get_dynamic_map")())());
                 dp.insert(boost::python::extract<string>(props[i][0]),
                           DP_SMART_PTR<dynamic_property_map>(pmap));
@@ -560,7 +558,7 @@ void GraphInterface::write_to_file(string file, boost::python::object pfile,
             {
                 // vertex indices must be between the [0, HardNumVertices(g)] range
                 vector_property_map<size_t> index_map(_vertex_index);
-                run_action<>()
+                run_action<>(false)
                     (*this,
                      [&](auto&& graph)
                      {
@@ -570,7 +568,7 @@ void GraphInterface::write_to_file(string file, boost::python::object pfile,
                 if (format == "dot")
                     graphviz_insert_index(dp, index_map);
 
-                run_action<>()
+                run_action<>(false)
                     (*this,
                      [&](auto&& graph)
                      {
@@ -584,7 +582,7 @@ void GraphInterface::write_to_file(string file, boost::python::object pfile,
                 if (format == "dot")
                     graphviz_insert_index(dp, _vertex_index);
 
-                run_action<>()
+                run_action<>(false)
                     (*this,
                      [&](auto&& graph)
                      {
