@@ -1,6 +1,6 @@
 // graph-tool -- a general graph modification and manipulation thingy
 //
-// Copyright (C) 2006-2023 Tiago de Paula Peixoto <tiago@skewed.de>
+// Copyright (C) 2006-2025 Tiago de Paula Peixoto <tiago@skewed.de>
 //
 // This program is free software; you can redistribute it and/or modify it under
 // the terms of the GNU Lesser General Public License as published by the Free
@@ -116,4 +116,6 @@ void export_openmp()
     def("openmp_set_num_threads", &openmp_set_num_threads);
     def("openmp_get_schedule", &openmp_get_schedule);
     def("openmp_set_schedule", &openmp_set_schedule);
+    def("openmp_get_thresh", &get_openmp_min_thresh);
+    def("openmp_set_thresh", &set_openmp_min_thresh);
 };
