@@ -1,6 +1,6 @@
 // graph-tool -- a general graph modification and manipulation thingy
 //
-// Copyright (C) 2006-2023 Tiago de Paula Peixoto <tiago@skewed.de>
+// Copyright (C) 2006-2025 Tiago de Paula Peixoto <tiago@skewed.de>
 //
 // This program is free software; you can redistribute it and/or modify it under
 // the terms of the GNU Lesser General Public License as published by the Free
@@ -58,17 +58,15 @@ struct SumOp
     template <class Graph, class Vertex, class EProp, class VProp>
     void operator()(Vertex v, EProp& eprop, VProp& vprop, Graph& g) const
     {
-        typedef typename property_traits<EProp>::value_type eval_t;
         typedef typename property_traits<VProp>::value_type vval_t;
 
-        convert<vval_t, eval_t> conv;
         size_t count = 0;
         for (const auto& e : out_edges_range(v, g))
         {
             if (count == 0)
-                vprop[v] = conv(eprop[e]);
+                vprop[v] = convert<vval_t>(eprop[e]);
             else
-                vprop[v] += conv(eprop[e]);
+                vprop[v] += convert<vval_t>(eprop[e]);
             ++count;
         }
     }
@@ -79,17 +77,15 @@ struct ProdOp
     template <class Graph, class Vertex, class EProp, class VProp>
     void operator()(Vertex v, EProp& eprop, VProp& vprop, Graph& g) const
     {
-        typedef typename property_traits<EProp>::value_type eval_t;
         typedef typename property_traits<VProp>::value_type vval_t;
 
-        convert<vval_t, eval_t> conv;
         size_t count = 0;
         for (const auto& e : out_edges_range(v, g))
         {
             if (count == 0)
-                vprop[v] = conv(eprop[e]);
+                vprop[v] = convert<vval_t>(eprop[e]);
             else
-                vprop[v] *= conv(eprop[e]);
+                vprop[v] *= convert<vval_t>(eprop[e]);
             ++count;
         }
     }
@@ -101,19 +97,16 @@ struct MinOp
     template <class Graph, class Vertex, class EProp, class VProp>
     void operator()(Vertex v, EProp& eprop, VProp& vprop, Graph& g) const
     {
-        typedef typename property_traits<EProp>::value_type eval_t;
         typedef typename property_traits<VProp>::value_type vval_t;
 
-        convert<vval_t, eval_t> conv;
-
         for (const auto& e : out_edges_range(v, g))
         {
-            vprop[v] = conv(eprop[e]);
+            vprop[v] = convert<vval_t>(eprop[e]);
             break;
         }
 
         for (const auto& e : out_edges_range(v, g))
-            vprop[v] = std::min(vprop[v], conv(eprop[e]));
+            vprop[v] = std::min(vprop[v], convert<vval_t>(eprop[e]));
     }
 };
 
@@ -122,42 +115,39 @@ struct MaxOp
     template <class Graph, class Vertex, class EProp, class VProp>
     void operator()(Vertex v, EProp& eprop, VProp& vprop, Graph& g) const
     {
-        typedef typename property_traits<EProp>::value_type eval_t;
         typedef typename property_traits<VProp>::value_type vval_t;
 
-        convert<vval_t, eval_t> conv;
-
         for (const auto& e : out_edges_range(v, g))
         {
-            vprop[v] = conv(eprop[e]);
+            vprop[v] = convert<vval_t>(eprop[e]);
             break;
         }
 
         for (const auto& e : out_edges_range(v, g))
-            vprop[v] = std::max(vprop[v], conv(eprop[e]));
+            vprop[v] = std::max(vprop[v], convert<vval_t>(eprop[e]));
     }
 };
 
 struct do_out_edges_op
 {
     template <class Graph, class EProp, class OP>
-    void operator()(Graph& g, EProp eprop, boost::any avprop, OP op) const
+    void operator()(Graph& g, EProp eprop, std::any avprop, OP op) const
     {
         typedef typename property_traits<EProp>::value_type eval_t;
-        typedef typename property_map_type::apply<
-            typename mpl::if_<std::is_same<eval_t, size_t>,
-                              int64_t, eval_t>::type,
-            GraphInterface::vertex_index_map_t>::type VProp;
+        typedef property_map_t<
+            std::conditional_t<std::is_same_v<eval_t, size_t>,
+                               int64_t, eval_t>,
+            GraphInterface::vertex_index_map_t> VProp;
 
-        auto vprop = boost::any_cast<VProp>(avprop).get_unchecked(num_vertices(g));
+        auto vprop = std::any_cast<VProp>(avprop).get_unchecked(num_vertices(g));
 
         #ifdef _OPENMP
         size_t __attribute__ ((unused)) nt = omp_get_num_threads();
-        if (std::is_convertible<eval_t,python::object>::value)
+        if (std::is_same<eval_t, python::object>::value)
             nt = 1; // python is not thread-safe
         #endif
 
-        #pragma omp parallel if (num_vertices(g) > OPENMP_MIN_THRESH) \
+        #pragma omp parallel if (num_vertices(g) > get_openmp_min_thresh()) \
             num_threads(nt)
         parallel_vertex_loop_no_spawn
             (g,
@@ -168,7 +158,7 @@ struct do_out_edges_op
     }
 };
 
-void out_edges_op(GraphInterface& gi, boost::any eprop, boost::any vprop,
+void out_edges_op(GraphInterface& gi, std::any eprop, std::any vprop,
                   std::string op)
 {
     if (op == "sum")
@@ -181,7 +171,7 @@ void out_edges_op(GraphInterface& gi, boost::any eprop, boost::any vprop,
                      (std::forward<decltype(graph)>(graph),
                       std::forward<decltype(a2)>(a2), vprop, SumOp());
              },
-             edge_properties())(eprop);
+             edge_properties)(eprop);
     }
     else if (op == "prod")
     {
@@ -193,7 +183,7 @@ void out_edges_op(GraphInterface& gi, boost::any eprop, boost::any vprop,
                      (std::forward<decltype(graph)>(graph),
                       std::forward<decltype(a2)>(a2), vprop, ProdOp());
              },
-             edge_properties())(eprop);
+             edge_properties)(eprop);
     }
     else if (op == "min")
     {
@@ -205,7 +195,7 @@ void out_edges_op(GraphInterface& gi, boost::any eprop, boost::any vprop,
                      (std::forward<decltype(graph)>(graph),
                       std::forward<decltype(a2)>(a2), vprop, MinOp());
              },
-             edge_properties())(eprop);
+             edge_properties)(eprop);
     }
     else if (op == "max")
     {
@@ -217,6 +207,6 @@ void out_edges_op(GraphInterface& gi, boost::any eprop, boost::any vprop,
                      (std::forward<decltype(graph)>(graph),
                       std::forward<decltype(a2)>(a2), vprop, MaxOp());
              },
-             edge_properties())(eprop);
+             edge_properties)(eprop);
     }
 }
