@@ -1,6 +1,6 @@
 // graph-tool -- a general graph modification and manipulation thingy
 //
-// Copyright (C) 2006-2023 Tiago de Paula Peixoto <tiago@skewed.de>
+// Copyright (C) 2006-2025 Tiago de Paula Peixoto <tiago@skewed.de>
 //
 // This program is free software; you can redistribute it and/or modify it under
 // the terms of the GNU Lesser General Public License as published by the Free
@@ -33,16 +33,16 @@ struct do_perfect_vhash
 {
     template <class Graph, class VertexPropertyMap, class HashProp>
     void operator()(Graph& g, VertexPropertyMap prop, HashProp hprop,
-                    boost::any& adict) const
+                    std::any& adict) const
     {
         typedef typename property_traits<VertexPropertyMap>::value_type val_t;
         typedef typename property_traits<HashProp>::value_type hash_t;
         typedef std::unordered_map<val_t, hash_t> dict_t;
 
-        if (adict.empty())
+        if (!adict.has_value())
             adict = dict_t();
 
-        dict_t& dict = any_cast<dict_t&>(adict);
+        dict_t& dict = std::any_cast<dict_t&>(adict);
 
         for (auto v : vertices_range(g))
         {
@@ -63,10 +63,10 @@ struct do_perfect_vhash
     }
 };
 
-void perfect_vhash(GraphInterface& gi, boost::any prop, boost::any hprop,
-                   boost::any& dict)
+void perfect_vhash(GraphInterface& gi, std::any prop, std::any hprop,
+                   std::any& dict)
 {
-    run_action<graph_tool::detail::always_directed>()
+    run_action<decltype(graph_tool::always_directed)>()
         (gi,
          [&](auto&& graph, auto&& a2, auto&& a3)
          {
@@ -75,7 +75,7 @@ void perfect_vhash(GraphInterface& gi, boost::any prop, boost::any hprop,
                   std::forward<decltype(a2)>(a2),
                   std::forward<decltype(a3)>(a3), dict);
          },
-         vertex_properties(), writable_vertex_scalar_properties())(prop, hprop);
+         vertex_properties, writable_vertex_scalar_properties)(prop, hprop);
 }
 
 struct do_set_vertex_property
@@ -86,15 +86,16 @@ struct do_set_vertex_property
         typedef typename property_traits<PropertyMap>::value_type val_t;
         val_t val = boost::python::extract<val_t>(oval);
 
+        GILRelease gil_release;
         for (auto v : vertices_range(g))
             prop[v] = val;
     }
 };
 
-void set_vertex_property(GraphInterface& gi, boost::any prop,
+void set_vertex_property(GraphInterface& gi, std::any prop,
                          boost::python::object val)
 {
-    run_action<>()
+    run_action<>(false)
         (gi,
          [&](auto&& graph, auto&& a2)
          {
@@ -102,7 +103,7 @@ void set_vertex_property(GraphInterface& gi, boost::any prop,
                  (std::forward<decltype(graph)>(graph),
                   std::forward<decltype(a2)>(a2), val);
          },
-         writable_vertex_properties())(prop);
+         writable_vertex_properties)(prop);
 }
 
 struct do_set_edge_property
@@ -113,15 +114,16 @@ struct do_set_edge_property
         typedef typename property_traits<PropertyMap>::value_type val_t;
         val_t val = boost::python::extract<val_t>(oval);
 
+        GILRelease gil_release;
         for (auto e : edges_range(g))
             prop[e] = val;
     }
 };
 
-void set_edge_property(GraphInterface& gi, boost::any prop,
+void set_edge_property(GraphInterface& gi, std::any prop,
                        boost::python::object val)
 {
-    run_action<>()
+    run_action<>(false)
         (gi,
          [&](auto&& graph, auto&& a2)
          {
@@ -129,5 +131,5 @@ void set_edge_property(GraphInterface& gi, boost::any prop,
                  (std::forward<decltype(graph)>(graph),
                   std::forward<decltype(a2)>(a2), val);
          },
-         writable_edge_properties())(prop);
+         writable_edge_properties)(prop);
 }
