@@ -1,6 +1,6 @@
 // graph-tool -- a general graph modification and manipulation thingy
 //
-// Copyright (C) 2006-2023 Tiago de Paula Peixoto <tiago@skewed.de>
+// Copyright (C) 2006-2025 Tiago de Paula Peixoto <tiago@skewed.de>
 //
 // This program is free software; you can redistribute it and/or modify it under
 // the terms of the GNU Lesser General Public License as published by the Free
@@ -31,16 +31,16 @@ struct do_perfect_ehash
 {
     template <class Graph, class EdgePropertyMap, class HashProp>
     void operator()(Graph& g, EdgePropertyMap prop, HashProp hprop,
-                    boost::any& adict) const
+                    std::any& adict) const
     {
         typedef typename property_traits<EdgePropertyMap>::value_type val_t;
         typedef typename property_traits<HashProp>::value_type hash_t;
         typedef unordered_map<val_t, hash_t> dict_t;
 
-        if (adict.empty())
+        if (!adict.has_value())
             adict = dict_t();
 
-        dict_t& dict = any_cast<dict_t&>(adict);
+        dict_t& dict = std::any_cast<dict_t&>(adict);
 
         for (auto e : edges_range(g))
         {
@@ -61,10 +61,10 @@ struct do_perfect_ehash
     }
 };
 
-void perfect_ehash(GraphInterface& gi, boost::any prop, boost::any hprop,
-                   boost::any& dict)
+void perfect_ehash(GraphInterface& gi, std::any prop, std::any hprop,
+                   std::any& dict)
 {
-    run_action<graph_tool::detail::always_directed>()
+    run_action<decltype(graph_tool::always_directed)>()
         (gi,
          [&](auto&& graph, auto&& a2, auto&& a3)
          {
@@ -73,5 +73,5 @@ void perfect_ehash(GraphInterface& gi, boost::any prop, boost::any hprop,
                   std::forward<decltype(a2)>(a2),
                   std::forward<decltype(a3)>(a3), dict);
          },
-         edge_properties(), writable_edge_scalar_properties())(prop, hprop);
+         edge_properties, writable_edge_scalar_properties)(prop, hprop);
 }
