@@ -1,6 +1,6 @@
 // graph-tool -- a general graph modification and manipulation thingy
 //
-// Copyright (C) 2006-2023 Tiago de Paula Peixoto <tiago@skewed.de>
+// Copyright (C) 2006-2025 Tiago de Paula Peixoto <tiago@skewed.de>
 //
 // This program is free software; you can redistribute it and/or modify it under
 // the terms of the GNU Lesser General Public License as published by the Free
@@ -28,16 +28,16 @@ using namespace std;
 using namespace boost;
 using namespace graph_tool;
 
-void edge_property_map_values(GraphInterface& g, boost::any src_prop,
-                              boost::any tgt_prop, boost::python::object mapper);
+void edge_property_map_values(GraphInterface& g, std::any src_prop,
+                              std::any tgt_prop, boost::python::object mapper);
 
-void property_map_values(GraphInterface& g, boost::any src_prop,
-                         boost::any tgt_prop, boost::python::object mapper,
+void property_map_values(GraphInterface& g, std::any src_prop,
+                         std::any tgt_prop, boost::python::object mapper,
                          bool edge)
 {
     if (!edge)
     {
-        run_action<graph_tool::detail::always_directed_never_reversed>()
+        run_action<decltype(graph_tool::always_directed_never_reversed)>(false)
             (g,
              [&](auto&& graph, auto&& a2, auto&& a3)
              {
@@ -46,8 +46,8 @@ void property_map_values(GraphInterface& g, boost::any src_prop,
                       std::forward<decltype(a2)>(a2),
                       std::forward<decltype(a3)>(a3), mapper);
              },
-             vertex_properties(),
-             writable_vertex_properties())(src_prop, tgt_prop);
+             vertex_properties,
+             writable_vertex_properties)(src_prop, tgt_prop);
     }
     else
     {
