@@ -1,6 +1,6 @@
 // graph-tool -- a general graph modification and manipulation thingy
 //
-// Copyright (C) 2006-2023 Tiago de Paula Peixoto <tiago@skewed.de>
+// Copyright (C) 2006-2025 Tiago de Paula Peixoto <tiago@skewed.de>
 //
 // This program is free software; you can redistribute it and/or modify it under
 // the terms of the GNU Lesser General Public License as published by the Free
@@ -29,10 +29,10 @@ using namespace boost;
 using namespace graph_tool;
 
 
-void edge_property_map_values(GraphInterface& g, boost::any src_prop,
-                              boost::any tgt_prop, boost::python::object mapper)
+void edge_property_map_values(GraphInterface& g, std::any src_prop,
+                              std::any tgt_prop, boost::python::object mapper)
 {
-    run_action<graph_tool::detail::always_directed_never_reversed>()
+    run_action<decltype(graph_tool::always_directed_never_reversed)>(false)
         (g,
          [&](auto&& graph, auto&& a2, auto&& a3)
          {
@@ -41,5 +41,5 @@ void edge_property_map_values(GraphInterface& g, boost::any src_prop,
                   std::forward<decltype(a2)>(a2),
                   std::forward<decltype(a3)>(a3), mapper);
          },
-         edge_properties(), writable_edge_properties())(src_prop, tgt_prop);
+         edge_properties, writable_edge_properties)(src_prop, tgt_prop);
 }
