@@ -1,6 +1,6 @@
 // graph-tool -- a general graph modification and manipulation thingy
 //
-// Copyright (C) 2006-2023 Tiago de Paula Peixoto <tiago@skewed.de>
+// Copyright (C) 2006-2025 Tiago de Paula Peixoto <tiago@skewed.de>
 //
 // This program is free software; you can redistribute it and/or modify it under
 // the terms of the GNU Lesser General Public License as published by the Free
@@ -59,7 +59,7 @@ python::object do_multicanonical_sweep(python::object omulticanonical_state,
                  typedef typename std::remove_reference<decltype(mcmc_state)>::type
                      mcmc_state_t;
 
-                 omulticanonical_state.attr("state") = boost::any(mcmc_state);
+                 omulticanonical_state.attr("state") = std::any(mcmc_state);
 
                  multicanonical_block_state<mcmc_state_t>::make_dispatch
                      (omulticanonical_state,
