@@ -1,6 +1,6 @@
 // graph-tool -- a general graph modification and manipulation thingy
 //
-// Copyright (C) 2006-2023 Tiago de Paula Peixoto <tiago@skewed.de>
+// Copyright (C) 2006-2025 Tiago de Paula Peixoto <tiago@skewed.de>
 //
 // This program is free software; you can redistribute it and/or modify it under
 // the terms of the GNU Lesser General Public License as published by the Free
@@ -26,30 +26,32 @@
 using namespace graph_tool;
 
 boost::python::object
-clique_iter_mh(GraphInterface& gi, boost::any ac, boost::any ax,
-               boost::any ais_fac, boost::any ais_max, boost::python::object oEd,
+clique_iter_mh(GraphInterface& gi, std::any ac, std::any ax,
+               std::any ais_fac, std::any ais_max, boost::python::object oEd,
                int N, int E, double beta, size_t niter, rng_t& rng)
 {
-    typedef typename vprop_map_t<int32_t>::type vprop_t;
-    typedef typename vprop_map_t<uint8_t>::type vbprop_t;
-    typedef typename vprop_map_t<std::vector<int32_t>>::type vvprop_t;
+    typedef vprop_map_t<int32_t> vprop_t;
+    typedef vprop_map_t<uint8_t> vbprop_t;
+    typedef vprop_map_t<std::vector<int32_t>> vvprop_t;
 
-    vprop_t x = boost::any_cast<vprop_t>(ax);
-    vvprop_t c = boost::any_cast<vvprop_t>(ac);
-    vbprop_t is_fac = boost::any_cast<vbprop_t>(ais_fac);
-    vbprop_t is_max = boost::any_cast<vbprop_t>(ais_max);
+    vprop_t x = std::any_cast<vprop_t>(ax);
+    vvprop_t c = std::any_cast<vvprop_t>(ac);
+    vbprop_t is_fac = std::any_cast<vbprop_t>(ais_fac);
+    vbprop_t is_max = std::any_cast<vbprop_t>(ais_max);
 
     multi_array_ref<int32_t, 1> Ed = get_array<int32_t, 1>(oEd);
 
     boost::python::object pret;
-    gt_dispatch<>()
+    gt_dispatch<>(false)
         ([&](auto& g){
+             GILRelease gil_release;
              auto ret = iter_mh(g, x.get_unchecked(), c.get_unchecked(),
                                 is_fac.get_unchecked(), is_max.get_unchecked(),
                                 Ed, N, E, beta, niter, rng);
+             gil_release.restore();
              pret = boost::python::make_tuple(get<0>(ret), get<1>(ret));
          },
-         all_graph_views())
+         all_graph_views)
         (gi.get_graph_view());
     return pret;
 };
