@@ -1,6 +1,6 @@
 // graph-tool -- a general graph modification and manipulation thingy
 //
-// Copyright (C) 2006-2023 Tiago de Paula Peixoto <tiago@skewed.de>
+// Copyright (C) 2006-2025 Tiago de Paula Peixoto <tiago@skewed.de>
 //
 // This program is free software; you can redistribute it and/or modify it under
 // the terms of the GNU Lesser General Public License as published by the Free
@@ -129,21 +129,21 @@ python::object multiflip_mcmc_layered_overlap_sweep_parallel(python::object omcm
         overlap_block_state::dispatch(dispatch);
     }
 
-    parallel_rng<rng_t>::init(rng);
+    parallel_rng<rng_t> prng(rng);
 
     std::vector<std::tuple<double, size_t, size_t>> rets(N);
 
     #pragma omp parallel for schedule(runtime)
     for (size_t i = 0; i < N; ++i)
     {
-        auto& rng_ = parallel_rng<rng_t>::get(rng);
+        auto& rng_ = prng.get(rng);
         rets[i] = sweeps[i]->run(rng_);
     }
 
     python::list orets;
     for (auto& ret : rets)
         orets.append(tuple_apply([&](auto&... args){ return python::make_tuple(args...); }, ret));
-    return std::move(orets);
+    return orets;
 }
 
 #define __MOD__ inference
