@@ -1,6 +1,6 @@
 // graph-tool -- a general graph modification and manipulation thingy
 //
-// Copyright (C) 2006-2023 Tiago de Paula Peixoto <tiago@skewed.de>
+// Copyright (C) 2006-2025 Tiago de Paula Peixoto <tiago@skewed.de>
 //
 // This program is free software; you can redistribute it and/or modify it under
 // the terms of the GNU Lesser General Public License as published by the Free
@@ -72,7 +72,7 @@ REGISTER_MOD
                            auto bv = get_bv(ob);
                            state.add_partition(bv, r, relabel);
                        })
-                 .def("entropy", &state_t::entropy)
+                 .def("entropy", +[](state_t& state){ return state.entropy(); })
                  .def("posterior_entropy", &state_t::posterior_entropy)
                  .def("posterior_lprob",
                       +[](state_t& state, size_t r, object obv, bool MLE)
