@@ -1,6 +1,6 @@
 // graph-tool -- a general graph modification and manipulation thingy
 //
-// Copyright (C) 2006-2023 Tiago de Paula Peixoto <tiago@skewed.de>
+// Copyright (C) 2006-2025 Tiago de Paula Peixoto <tiago@skewed.de>
 //
 // This program is free software; you can redistribute it and/or modify it under
 // the terms of the GNU Lesser General Public License as published by the Free
@@ -16,7 +16,6 @@
 // along with this program. If not, see <http://www.gnu.org/licenses/>.
 
 #include "graph_tool.hh"
-#include "random.hh"
 
 #include <boost/python.hpp>
 
@@ -59,7 +58,8 @@ REGISTER_MOD
                    no_init);
              c.def("move_vertex", move_vertex)
                  .def("virtual_move", virtual_move)
-                 .def("entropy", &state_t::entropy);
+                 .def("entropy", +[](state_t& state,
+                                     const pp_entropy_args_t& ea){ return state.entropy(ea); });
          });
 
     class_<pp_entropy_args_t>("pp_entropy_args")
