@@ -1,6 +1,6 @@
 // graph-tool -- a general graph modification and manipulation thingy
 //
-// Copyright (C) 2006-2023 Tiago de Paula Peixoto <tiago@skewed.de>
+// Copyright (C) 2006-2025 Tiago de Paula Peixoto <tiago@skewed.de>
 //
 // This program is free software; you can redistribute it and/or modify it under
 // the terms of the GNU Lesser General Public License as published by the Free
@@ -20,7 +20,7 @@
 
 #include <boost/python.hpp>
 
-#define GRAPH_VIEWS never_filtered_never_reversed
+//#define GRAPH_VIEWS never_filtered_never_reversed
 #include "../blockmodel/graph_blockmodel.hh"
 
 #include "graph_ranked.hh"
