@@ -1,6 +1,6 @@
 // graph-tool -- a general graph modification and manipulation thingy
 //
-// Copyright (C) 2006-2023 Tiago de Paula Peixoto <tiago@skewed.de>
+// Copyright (C) 2006-2025 Tiago de Paula Peixoto <tiago@skewed.de>
 //
 // This program is free software; you can redistribute it and/or modify it under
 // the terms of the GNU Lesser General Public License as published by the Free
@@ -45,10 +45,13 @@ void clear_lgamma()
 
 void init_cache()
 {
-    auto nt = get_num_threads();
-    __lgamma_cache.resize(nt);
-    __xlogx_cache.resize(nt);
-    __safelog_cache.resize(nt);
+    size_t nt = get_num_threads();
+    if (__lgamma_cache.size() < nt)
+    {
+        __lgamma_cache.resize(nt);
+        __xlogx_cache.resize(nt);
+        __safelog_cache.resize(nt);
+    }
 }
 
 
