@@ -1,6 +1,6 @@
 // graph-tool -- a general graph modification and manipulation thingy
 //
-// Copyright (C) 2006-2023 Tiago de Paula Peixoto <tiago@skewed.de>
+// Copyright (C) 2006-2025 Tiago de Paula Peixoto <tiago@skewed.de>
 //
 // This program is free software; you can redistribute it and/or modify it under
 // the terms of the GNU Lesser General Public License as published by the Free
@@ -25,27 +25,25 @@ using namespace std;
 using namespace boost;
 using namespace graph_tool;
 
-void arf_layout(GraphInterface& g, boost::any pos, boost::any weight, double d,
+void arf_layout(GraphInterface& g, std::any pos, std::any weight, double d,
                 double a, double dt, size_t max_iter, double epsilon,
                 size_t dim)
 {
     typedef UnityPropertyMap<int,GraphInterface::edge_t> weight_map_t;
-    typedef boost::mpl::push_back<edge_scalar_properties, weight_map_t>::type
-        edge_props_t;
+    auto edge_props_t = hana::append(edge_scalar_properties,
+                                     hana::type<weight_map_t>());
 
-    if(weight.empty())
+    if(!weight.has_value())
         weight = weight_map_t();
-    run_action<graph_tool::detail::never_directed>()
+
+    run_action<decltype(graph_tool::never_directed), false>()
         (g,
-         [&](auto&& graph, auto&& a2, auto&& a3)
+         [&](auto& g, auto pos, auto weight)
          {
-             return get_arf_layout()
-                 (std::forward<decltype(graph)>(graph),
-                  std::forward<decltype(a2)>(a2),
-                  std::forward<decltype(a3)>(a3), a, d, dt, epsilon, max_iter,
-                  dim);
+             return get_arf_layout()(g, pos, weight, a, d, dt, epsilon,
+                                     max_iter, dim);
          },
-         vertex_floating_vector_properties(), edge_props_t())(pos, weight);
+         vertex_floating_vector_properties, edge_props_t)(pos, weight);
 }
 
 #include <boost/python.hpp>
