@@ -1,6 +1,6 @@
 // graph-tool -- a general graph modification and manipulation thingy
 //
-// Copyright (C) 2006-2023 Tiago de Paula Peixoto <tiago@skewed.de>
+// Copyright (C) 2006-2025 Tiago de Paula Peixoto <tiago@skewed.de>
 //
 // This program is free software; you can redistribute it and/or modify it under
 // the terms of the GNU Lesser General Public License as published by the Free
@@ -41,7 +41,7 @@ namespace graph_tool
 {
 // convert point types
 template <class Val>
-struct convert<vector<Val>, typename convex_topology<2>::point>
+struct convert_dispatch<vector<Val>, typename convex_topology<2>::point>
 {
     vector<Val> operator()(const typename convex_topology<2>::point& p) const
     {
@@ -53,7 +53,7 @@ struct convert<vector<Val>, typename convex_topology<2>::point>
 };
 
 template <class Val>
-struct convert<typename convex_topology<2>::point, vector<Val> >
+struct convert_dispatch<typename convex_topology<2>::point, vector<Val> >
 {
     typename convex_topology<2>::point operator()(const vector<Val>& v) const
     {
@@ -65,13 +65,24 @@ struct convert<typename convex_topology<2>::point, vector<Val> >
 };
 } // graph_tool namespace
 
+namespace boost
+{
+template <>
+std::string lexical_cast(const typename convex_topology<2>::point& p)
+{
+    std::string out = "(" + boost::lexical_cast<std::string>(p[0]) + ", " +
+        boost::lexical_cast<std::string>(p[1]) + ")";
+    return out;
+}
+}
+
 template<class T>
 struct anneal_cooling
 {
     typedef T result_type;
 
     anneal_cooling(T ti, T tf, std::size_t iterations)
-        : _ti(ti), _tf(tf), _iter(0), _n_iter(iterations) 
+        : _ti(ti), _tf(tf), _iter(0), _n_iter(iterations)
     {
         _beta = (log(_tf) - log(_ti)) / _n_iter;
     }
@@ -153,41 +164,37 @@ struct get_layout
 };
 
 
-void fruchterman_reingold_layout(GraphInterface& g, boost::any pos,
-                                 boost::any weight, double a, double r,
+void fruchterman_reingold_layout(GraphInterface& g, std::any pos,
+                                 std::any weight, double a, double r,
                                  bool square, double scale, bool grid,
                                  double ti, double tf, size_t max_iter)
 {
     typedef UnityPropertyMap<int,GraphInterface::edge_t> weight_map_t;
-    typedef boost::mpl::push_back<edge_scalar_properties, weight_map_t>::type
-        edge_props_t;
+    auto edge_props_t = hana::append(edge_scalar_properties,
+                                     hana::type<weight_map_t>());
 
-    if(weight.empty())
+    if(!weight.has_value())
         weight = weight_map_t();
     if (square)
-        run_action<graph_tool::detail::never_directed>()
+        run_action<decltype(graph_tool::never_directed)>()
             (g,
-             [&](auto&& graph, auto&& a2, auto&& a3)
+             [&](auto&& g, auto pos, auto weight)
              {
-                 return get_layout<square_topology<>>()(
-                     std::forward<decltype(graph)>(graph),
-                     std::forward<decltype(a2)>(a2),
-                     std::forward<decltype(a3)>(a3), make_pair(a, r), scale,
-                     grid, make_pair(ti, tf), max_iter);
+                 return get_layout<square_topology<>>()
+                     (g, pos, weight, make_pair(a, r), scale,
+                      grid, make_pair(ti, tf), max_iter);
              },
-             vertex_floating_vector_properties(), edge_props_t())(pos, weight);
+             vertex_floating_vector_properties, edge_props_t)(pos, weight);
     else
-        run_action<graph_tool::detail::never_directed>()
+        run_action<decltype(graph_tool::never_directed)>()
             (g,
-             [&](auto&& graph, auto&& a2, auto&& a3)
+             [&](auto& g, auto pos, auto weight)
              {
-                 return get_layout<circle_topology<>>()(
-                     std::forward<decltype(graph)>(graph),
-                     std::forward<decltype(a2)>(a2),
-                     std::forward<decltype(a3)>(a3), make_pair(a, r), scale,
-                     grid, make_pair(ti, tf), max_iter);
+                 return get_layout<circle_topology<>>()
+                     (g, pos, weight, make_pair(a, r), scale,
+                      grid, make_pair(ti, tf), max_iter);
              },
-             vertex_floating_vector_properties(), edge_props_t())(pos, weight);
+             vertex_floating_vector_properties, edge_props_t)(pos, weight);
 }
 
 #include <boost/python.hpp>
