@@ -1,6 +1,6 @@
 // graph-tool -- a general graph modification and manipulation thingy
 //
-// Copyright (C) 2006-2023 Tiago de Paula Peixoto <tiago@skewed.de>
+// Copyright (C) 2006-2025 Tiago de Paula Peixoto <tiago@skewed.de>
 //
 // This program is free software; you can redistribute it and/or modify it under
 // the terms of the GNU Lesser General Public License as published by the Free
@@ -31,62 +31,63 @@ using namespace std;
 using namespace boost;
 using namespace graph_tool;
 
-void sfdp_layout(GraphInterface& g, boost::any pos, boost::any vweight,
-                 boost::any eweight, boost::any pin, python::object spring_parms,
+void sfdp_layout(GraphInterface& g, std::any pos, std::any vweight,
+                 std::any eweight, std::any pin, python::object spring_parms,
                  double theta, double init_step, double step_schedule,
                  size_t max_level, double epsilon, size_t max_iter,
                  bool adaptive, bool verbose, rng_t& rng)
 {
     typedef UnityPropertyMap<int,GraphInterface::vertex_t> vweight_map_t;
     typedef UnityPropertyMap<int,GraphInterface::edge_t> eweight_map_t;
-    typedef mpl::push_back<vertex_scalar_properties, vweight_map_t>::type
-        vertex_props_t;
-    typedef mpl::push_back<edge_scalar_properties, eweight_map_t>::type
-        edge_props_t;
+    auto vertex_props_t = hana::append(vertex_scalar_properties,
+                                       hana::type<vweight_map_t>());
+    auto edge_props_t = hana::append(edge_scalar_properties,
+                                     hana::type<eweight_map_t>());
 
     double C = python::extract<double>(spring_parms[0]);
     double K = python::extract<double>(spring_parms[1]);
     double p = python::extract<double>(spring_parms[2]);
-    double gamma = python::extract<double>(spring_parms[3]);
-    double mu = python::extract<double>(spring_parms[4]);
-    double kappa = python::extract<double>(spring_parms[5]);
+    std::vector<double> gamma;
+    for (int i = 0; i < python::len(spring_parms[3]); ++i)
+        gamma.push_back(python::extract<double>(spring_parms[3][i]));
 
     typedef multi_array_ref<int32_t,1> group_map_t;
     std::vector<group_map_t> groups;
-    for (int i = 0; i < python::len(spring_parms[6]); ++i)
-        groups.push_back(get_array<int32_t,1>(spring_parms[6][i]));
+    for (int i = 0; i < python::len(spring_parms[4]); ++i)
+        groups.push_back(get_array<int32_t,1>(spring_parms[4][i]));
 
-    double r = python::extract<double>(spring_parms[7]);
-    typedef vprop_map_t<int32_t>::type c_map_t;
-    c_map_t c_map = boost::any_cast<c_map_t>(python::extract<any&>(spring_parms[8])());
+    double r = python::extract<double>(spring_parms[5]);
+    size_t kc = python::extract<double>(spring_parms[6]);
+    typedef vprop_map_t<int32_t> c_map_t;
+    c_map_t c_map = std::any_cast<c_map_t>(python::extract<any&>(spring_parms[7])());
 
-    double R = python::extract<double>(spring_parms[9]);
-    typedef vprop_map_t<double>::type o_map_t;
-    o_map_t o_map = boost::any_cast<o_map_t>(python::extract<any&>(spring_parms[10])());
+    double R = python::extract<double>(spring_parms[8]);
+    typedef vprop_map_t<double> o_map_t;
+    o_map_t o_map = std::any_cast<o_map_t>(python::extract<any&>(spring_parms[9])());
 
-    if(vweight.empty())
+    if (!vweight.has_value())
         vweight = vweight_map_t();
-    if(eweight.empty())
+    if (!eweight.has_value())
         eweight = eweight_map_t();
 
-    typedef vprop_map_t<uint8_t>::type pin_map_t;
-    pin_map_t pin_map = boost::any_cast<pin_map_t>(pin);
+    typedef vprop_map_t<uint8_t> pin_map_t;
+    pin_map_t pin_map = std::any_cast<pin_map_t>(pin);
 
-    run_action<graph_tool::detail::never_directed>()
+    run_action<decltype(graph_tool::never_directed)>()
         (g,
          [&](auto&& graph, auto&& pos, auto&& vweight, auto&& eweight)
          {
              return get_sfdp_layout(graph, pos, vweight, eweight,
                                     pin_map.get_unchecked(num_vertices(g.get_graph())),
-                                    groups, C, K, p, theta, gamma, mu, kappa, r,
+                                    groups, C, K, p, theta, gamma, r, kc,
                                     c_map.get_unchecked(num_vertices(g.get_graph())),
                                     R, o_map.get_unchecked(num_vertices(g.get_graph())),
                                     init_step,
                                     step_schedule, max_level, epsilon, max_iter,
                                     adaptive, verbose, rng);
          },
-         vertex_floating_vector_properties(), vertex_props_t(),
-         edge_props_t())(pos, vweight, eweight);
+         vertex_floating_vector_properties, vertex_props_t,
+         edge_props_t)(pos, vweight, eweight);
 }
 
 struct do_propagate_pos
@@ -94,13 +95,13 @@ struct do_propagate_pos
     template <class Graph, class CoarseGraph, class VertexMap, class PosMap,
               class RNG>
     void operator()(Graph& g, CoarseGraph& cg, VertexMap vmap,
-                    boost::any acvmap, PosMap pos, boost::any acpos,
+                    std::any acvmap, PosMap pos, std::any acpos,
                     double delta, RNG& rng) const
     {
         typename PosMap::checked_t cpos =
-            boost::any_cast<typename PosMap::checked_t>(acpos);
+            std::any_cast<typename PosMap::checked_t>(acpos);
         typename VertexMap::checked_t cvmap =
-            boost::any_cast<typename VertexMap::checked_t>(acvmap);
+            std::any_cast<typename VertexMap::checked_t>(acvmap);
         typedef typename property_traits<VertexMap>::value_type c_t;
         typedef typename property_traits<PosMap>::value_type pos_t;
         typedef typename pos_t::value_type val_t;
@@ -124,14 +125,11 @@ struct do_propagate_pos
     }
 };
 
-void propagate_pos(GraphInterface& gi, GraphInterface& cgi, boost::any vmap,
-                   boost::any cvmap, boost::any pos, boost::any cpos,
+void propagate_pos(GraphInterface& gi, GraphInterface& cgi, std::any vmap,
+                   std::any cvmap, std::any pos, std::any cpos,
                    double delta, rng_t& rng)
 {
-    typedef mpl::vector<property_map_type::apply
-                            <int32_t,
-                             GraphInterface::vertex_index_map_t>::type>::type
-        vmaps_t;
+    auto vmaps_t = hana::tuple_t<vprop_map_t<int32_t>>;
 
     gt_dispatch<>()
         (
@@ -143,8 +141,8 @@ void propagate_pos(GraphInterface& gi, GraphInterface& cgi, boost::any vmap,
                      std::forward<decltype(a3)>(a3), cvmap,
                      std::forward<decltype(a4)>(a4), cpos, delta, rng);
             },
-            all_graph_views(), all_graph_views(), vmaps_t(),
-            vertex_floating_vector_properties())(
+            all_graph_views, all_graph_views, vmaps_t,
+            vertex_floating_vector_properties)(
             gi.get_graph_view(), cgi.get_graph_view(), vmap, pos);
 }
 
@@ -196,7 +194,7 @@ struct do_propagate_pos_mivs
 };
 
 
-void propagate_pos_mivs(GraphInterface& gi, boost::any mivs, boost::any pos,
+void propagate_pos_mivs(GraphInterface& gi, std::any mivs, std::any pos,
                         double delta, rng_t& rng)
 {
     run_action<>()
@@ -208,8 +206,8 @@ void propagate_pos_mivs(GraphInterface& gi, boost::any mivs, boost::any pos,
                   std::forward<decltype(a2)>(a2),
                   std::forward<decltype(a3)>(a3), delta, rng);
          },
-         vertex_scalar_properties(),
-         vertex_floating_vector_properties())(mivs, pos);
+         vertex_scalar_properties,
+         vertex_floating_vector_properties)(mivs, pos);
 }
 
 
@@ -220,7 +218,7 @@ struct do_avg_dist
     {
         size_t count = 0;
         double d = 0;
-        #pragma omp parallel if (num_vertices(g) > OPENMP_MIN_THRESH) \
+        #pragma omp parallel if (num_vertices(g) > get_openmp_min_thresh()) \
             reduction(+: d, count)
         parallel_vertex_loop_no_spawn
                 (g,
@@ -239,7 +237,7 @@ struct do_avg_dist
 };
 
 
-double avg_dist(GraphInterface& gi, boost::any pos)
+double avg_dist(GraphInterface& gi, std::any pos)
 {
     double d;
     run_action<>()
@@ -250,37 +248,24 @@ double avg_dist(GraphInterface& gi, boost::any pos)
                  (std::forward<decltype(graph)>(graph),
                   std::forward<decltype(a2)>(a2), d);
          },
-         vertex_scalar_vector_properties())(pos);
+         vertex_scalar_vector_properties)(pos);
     return d;
 }
 
-
-struct do_sanitize_pos
-{
-    template <class Graph, class PosMap>
-    void operator()(Graph& g, PosMap pos) const
-    {
-        parallel_vertex_loop
-                (g,
-                 [&](auto v)
-                 {
-                     pos[v].resize(2);
-                 });
-    }
-};
-
-
-void sanitize_pos(GraphInterface& gi, boost::any pos)
+void sanitize_pos(GraphInterface& gi, std::any apos)
 {
     run_action<>()
         (gi,
-         [&](auto&& graph, auto&& a2)
+         [&](auto&& g, auto&& pos)
          {
-             return do_sanitize_pos()
-                 (std::forward<decltype(graph)>(graph),
-                  std::forward<decltype(a2)>(a2));
+             parallel_vertex_loop
+                 (g,
+                  [&](auto v)
+                  {
+                      pos[v].resize(2);
+                  });
          },
-         vertex_scalar_vector_properties())(pos);
+         vertex_scalar_vector_properties)(apos);
 }
 
 #include <boost/python.hpp>
