@@ -1,6 +1,6 @@
 // graph-tool -- a general graph modification and manipulation thingy
 //
-// Copyright (C) 2006-2023 Tiago de Paula Peixoto <tiago@skewed.de>
+// Copyright (C) 2006-2025 Tiago de Paula Peixoto <tiago@skewed.de>
 //
 // This program is free software; you can redistribute it and/or modify it under
 // the terms of the GNU Lesser General Public License as published by the Free
@@ -31,13 +31,13 @@ using namespace std;
 using namespace boost;
 using namespace graph_tool;
 
-void incidence(GraphInterface& g, boost::any vindex, boost::any eindex,
+void incidence(GraphInterface& g, std::any vindex, std::any eindex,
                python::object odata, python::object oi,
                python::object oj)
 {
-    if (!belongs<vertex_scalar_properties>()(vindex))
+    if (!belongs(vertex_scalar_properties, vindex))
         throw ValueException("index vertex property must have a scalar value type");
-    if (!belongs<edge_scalar_properties>()(eindex))
+    if (!belongs(edge_scalar_properties, eindex))
         throw ValueException("index edge property must have a scalar value type");
 
     multi_array_ref<double,1> data = get_array<double,1>(odata);
@@ -52,13 +52,13 @@ void incidence(GraphInterface& g, boost::any vindex, boost::any eindex,
                   std::forward<decltype(a2)>(a2),
                   std::forward<decltype(a3)>(a3), data, i, j);
          },
-         vertex_scalar_properties(), edge_scalar_properties())(vindex, eindex);
+         vertex_scalar_properties, edge_scalar_properties)(vindex, eindex);
 }
 
-void incidence_matvec(GraphInterface& g, boost::any vindex, boost::any eindex,
+void incidence_matvec(GraphInterface& g, std::any vindex, std::any eindex,
                       python::object ov, python::object oret, bool transpose)
 {
-    if (!belongs<vertex_scalar_properties>()(vindex))
+    if (!belongs(vertex_scalar_properties, vindex))
         throw ValueException("index vertex property must have a scalar value type");
 
     multi_array_ref<double,1> v = get_array<double,1>(ov);
@@ -70,13 +70,13 @@ void incidence_matvec(GraphInterface& g, boost::any vindex, boost::any eindex,
          {
              return inc_matvec(graph, vi, ei, v, ret, transpose);
          },
-         vertex_scalar_properties(), edge_scalar_properties())(vindex, eindex);
+         vertex_scalar_properties, edge_scalar_properties)(vindex, eindex);
 }
 
-void incidence_matmat(GraphInterface& g, boost::any vindex, boost::any eindex,
+void incidence_matmat(GraphInterface& g, std::any vindex, std::any eindex,
                       python::object ov, python::object oret, bool transpose)
 {
-    if (!belongs<vertex_scalar_properties>()(vindex))
+    if (!belongs(vertex_scalar_properties, vindex))
         throw ValueException("index vertex property must have a scalar value type");
 
     multi_array_ref<double,2> v = get_array<double,2>(ov);
@@ -88,7 +88,7 @@ void incidence_matmat(GraphInterface& g, boost::any vindex, boost::any eindex,
          {
              return inc_matmat(graph, vi, ei, v, ret, transpose);
          },
-         vertex_scalar_properties(), edge_scalar_properties())(vindex, eindex);
+         vertex_scalar_properties, edge_scalar_properties)(vindex, eindex);
 }
 
 #define __MOD__ spectral
