@@ -1,6 +1,6 @@
 // graph-tool -- a general graph modification and manipulation thingy
 //
-// Copyright (C) 2006-2023 Tiago de Paula Peixoto <tiago@skewed.de>
+// Copyright (C) 2006-2025 Tiago de Paula Peixoto <tiago@skewed.de>
 //
 // This program is free software; you can redistribute it and/or modify it under
 // the terms of the GNU Lesser General Public License as published by the Free
@@ -31,22 +31,22 @@ using namespace std;
 using namespace boost;
 using namespace graph_tool;
 
-void laplacian(GraphInterface& g, boost::any index, boost::any weight,
+void laplacian(GraphInterface& g, std::any index, std::any weight,
                string sdeg, double r,
                python::object odata, python::object oi,
                python::object oj)
 {
-    if (!belongs<vertex_scalar_properties>()(index))
+    if (!belongs(vertex_scalar_properties, index))
         throw ValueException("index vertex property must have a scalar value type");
 
     typedef UnityPropertyMap<double, GraphInterface::edge_t> weight_map_t;
-    typedef mpl::push_back<edge_scalar_properties, weight_map_t>::type
-        weight_props_t;
+    auto weight_props_t = hana::append(edge_scalar_properties,
+                                       hana::type<weight_map_t>());
 
-    if (!weight.empty() && !belongs<edge_scalar_properties>()(weight))
+    if (weight.has_value() && !belongs(edge_scalar_properties, weight))
         throw ValueException("weight edge property must have a scalar value type");
 
-    if(weight.empty())
+    if(!weight.has_value())
         weight = weight_map_t();
 
     deg_t deg;
@@ -69,31 +69,31 @@ void laplacian(GraphInterface& g, boost::any index, boost::any weight,
                   std::forward<decltype(a2)>(a2),
                   std::forward<decltype(a3)>(a3), deg, r, data, i, j);
          },
-         vertex_scalar_properties(), weight_props_t())(index, weight);
+         vertex_scalar_properties, weight_props_t)(index, weight);
 }
 
-void laplacian_matvec(GraphInterface& g, boost::any index, boost::any weight,
-                      boost::any deg, double r, python::object ov,
+void laplacian_matvec(GraphInterface& g, std::any index, std::any weight,
+                      std::any deg, double r, python::object ov,
                       python::object oret)
 {
-    if (!belongs<vertex_scalar_properties>()(index))
+    if (!belongs(vertex_scalar_properties, index))
         throw ValueException("index vertex property must have a scalar value type");
 
     typedef UnityPropertyMap<double, GraphInterface::edge_t> weight_map_t;
-    typedef mpl::push_back<edge_scalar_properties, weight_map_t>::type
-        weight_props_t;
+    auto weight_props_t = hana::append(edge_scalar_properties,
+                                       hana::type<weight_map_t>());
 
-    if (!weight.empty() && !belongs<edge_scalar_properties>()(weight))
+    if (weight.has_value() && !belongs(edge_scalar_properties, weight))
         throw ValueException("weight edge property must have a scalar value type");
 
-    if(weight.empty())
+    if(!weight.has_value())
         weight = weight_map_t();
 
     multi_array_ref<double,1> v = get_array<double,1>(ov);
     multi_array_ref<double,1> ret = get_array<double,1>(oret);
 
-    typedef typename vprop_map_t<double>::type deg_t;
-    deg_t::unchecked_t d = any_cast<deg_t>(deg).get_unchecked();
+    typedef vprop_map_t<double> deg_t;
+    deg_t::unchecked_t d = std::any_cast<deg_t>(deg).get_unchecked();
 
     run_action<>()
         (g,
@@ -101,28 +101,28 @@ void laplacian_matvec(GraphInterface& g, boost::any index, boost::any weight,
          {
              return lap_matvec(graph, vi, w, d, r, v, ret);
          },
-         vertex_scalar_properties(), weight_props_t())(index, weight);
+         vertex_scalar_properties, weight_props_t)(index, weight);
 }
 
-void laplacian_matmat(GraphInterface& g, boost::any index, boost::any weight,
-                      boost::any deg, double r, python::object ov,
+void laplacian_matmat(GraphInterface& g, std::any index, std::any weight,
+                      std::any deg, double r, python::object ov,
                       python::object oret)
 {
-    if (!belongs<vertex_scalar_properties>()(index))
+    if (!belongs(vertex_scalar_properties, index))
         throw ValueException("index vertex property must have a scalar value type");
 
     typedef UnityPropertyMap<double, GraphInterface::edge_t> weight_map_t;
-    typedef mpl::push_back<edge_scalar_properties, weight_map_t>::type
-        weight_props_t;
+    auto weight_props_t = hana::append(edge_scalar_properties,
+                                       hana::type<weight_map_t>());
 
-    if (!weight.empty() && !belongs<edge_scalar_properties>()(weight))
+    if (weight.has_value() && !belongs(edge_scalar_properties, weight))
         throw ValueException("weight edge property must have a scalar value type");
 
-    if(weight.empty())
+    if(!weight.has_value())
         weight = weight_map_t();
 
-    typedef typename vprop_map_t<double>::type deg_t;
-    deg_t::unchecked_t d = any_cast<deg_t>(deg).get_unchecked();
+    typedef vprop_map_t<double> deg_t;
+    deg_t::unchecked_t d = std::any_cast<deg_t>(deg).get_unchecked();
 
     multi_array_ref<double,2> v = get_array<double,2>(ov);
     multi_array_ref<double,2> ret = get_array<double,2>(oret);
@@ -133,7 +133,7 @@ void laplacian_matmat(GraphInterface& g, boost::any index, boost::any weight,
          {
              return lap_matmat(graph, vi, w, d, r, v, ret);
          },
-         vertex_scalar_properties(), weight_props_t())(index, weight);
+         vertex_scalar_properties, weight_props_t)(index, weight);
 }
 
 #define __MOD__ spectral
