@@ -1,6 +1,6 @@
 // graph-tool -- a general graph modification and manipulation thingy
 //
-// Copyright (C) 2006-2023 Tiago de Paula Peixoto <tiago@skewed.de>
+// Copyright (C) 2006-2025 Tiago de Paula Peixoto <tiago@skewed.de>
 //
 // This program is free software; you can redistribute it and/or modify it under
 // the terms of the GNU Lesser General Public License as published by the Free
@@ -30,19 +30,19 @@ using namespace graph_tool;
 
 typedef Histogram<size_t, size_t, 1> hist_t;
 
-python::object sampled_distance_histogram(GraphInterface& gi, boost::any weight,
+python::object sampled_distance_histogram(GraphInterface& gi, std::any weight,
                                           const vector<long double>& bins,
                                           size_t n_samples, rng_t& rng)
 {
     python::object ret;
 
-    if (weight.empty())
+    if (!weight.has_value())
     {
-        run_action<>()
+        run_action<>(false)
             (gi,
              [&](auto&& graph)
              {
-                 return get_sampled_distance_histogram()
+                 get_sampled_distance_histogram()
                      (std::forward<decltype(graph)>(graph),
                       gi.get_vertex_index(), no_weightS(), n_samples, bins, ret,
                       rng);
@@ -50,16 +50,16 @@ python::object sampled_distance_histogram(GraphInterface& gi, boost::any weight,
     }
     else
     {
-        run_action<>()
+        run_action<>(false)
             (gi,
              [&](auto&& graph, auto&& a2)
              {
-                 return get_sampled_distance_histogram()
+                 get_sampled_distance_histogram()
                      (std::forward<decltype(graph)>(graph),
                       gi.get_vertex_index(), std::forward<decltype(a2)>(a2),
                       n_samples, bins, ret, rng);
              },
-             edge_scalar_properties())(weight);
+             edge_scalar_properties)(weight);
     }
     return ret;
 }
