@@ -1,6 +1,6 @@
 // graph-tool -- a general graph modification and manipulation thingy
 //
-// Copyright (C) 2006-2023 Tiago de Paula Peixoto <tiago@skewed.de>
+// Copyright (C) 2006-2025 Tiago de Paula Peixoto <tiago@skewed.de>
 //
 // This program is free software; you can redistribute it and/or modify it under
 // the terms of the GNU Lesser General Public License as published by the Free
@@ -111,7 +111,7 @@ struct do_all_pairs_search_unweighted
         typedef typename property_traits<DistMap>::value_type dist_t;
 
         vector<vertex_t> pred_map(num_vertices(g));
-        #pragma omp parallel if (num_vertices(g) > OPENMP_MIN_THRESH) \
+        #pragma omp parallel if (num_vertices(g) > get_openmp_min_thresh()) \
             firstprivate(pred_map)
         parallel_vertex_loop_no_spawn
                 (g,
@@ -126,10 +126,10 @@ struct do_all_pairs_search_unweighted
 };
 
 
-void get_all_dists(GraphInterface& gi, boost::any dist_map, boost::any weight,
+void get_all_dists(GraphInterface& gi, std::any dist_map, std::any weight,
                    bool dense)
 {
-    if (weight.empty())
+    if (!weight.has_value())
     {
         run_action<>()
             (gi,
@@ -139,7 +139,7 @@ void get_all_dists(GraphInterface& gi, boost::any dist_map, boost::any weight,
                      (std::forward<decltype(graph)>(graph),
                       std::forward<decltype(a2)>(a2));
              },
-             vertex_scalar_vector_properties())(dist_map);
+             vertex_scalar_vector_properties)(dist_map);
     }
     else
     {
@@ -152,8 +152,8 @@ void get_all_dists(GraphInterface& gi, boost::any dist_map, boost::any weight,
                       gi.get_vertex_index(), std::forward<decltype(a2)>(a2),
                       std::forward<decltype(a3)>(a3), dense);
              },
-             vertex_scalar_vector_properties(),
-             edge_scalar_properties())(dist_map, weight);
+             vertex_scalar_vector_properties,
+             edge_scalar_properties)(dist_map, weight);
     }
 }
 
