@@ -1,6 +1,6 @@
 // graph-tool -- a general graph modification and manipulation thingy
 //
-// Copyright (C) 2006-2023 Tiago de Paula Peixoto <tiago@skewed.de>
+// Copyright (C) 2006-2025 Tiago de Paula Peixoto <tiago@skewed.de>
 //
 // This program is free software; you can redistribute it and/or modify it under
 // the terms of the GNU Lesser General Public License as published by the Free
@@ -68,23 +68,24 @@ struct get_bipartite
     }
 };
 
-bool is_bipartite_dispatch(GraphInterface& gi, boost::any part_map, bool find_cycle,
+bool is_bipartite_dispatch(GraphInterface& gi, std::any part_map, bool find_cycle,
                            boost::python::list cycle)
 {
     bool is_bip;
     vector<size_t> vcycle;
 
-    if (part_map.empty())
+    if (!part_map.has_value())
         part_map = dummy_property_map();
 
-    typedef mpl::push_back<writable_vertex_scalar_properties,
-                           dummy_property_map>::type vertex_map_types;
+    auto vertex_map_types =
+        hana::append(writable_vertex_scalar_properties,
+                     hana::type<dummy_property_map>());
 
-    run_action<graph_tool::detail::never_directed>()
+    run_action<decltype(graph_tool::never_directed)>()
         (gi, std::bind(get_bipartite(), std::placeholders::_1,
                        gi.get_vertex_index(), std::placeholders::_2,
                        std::ref(is_bip), find_cycle, std::ref(vcycle)),
-         vertex_map_types())(part_map);
+         vertex_map_types)(part_map);
 
     for (auto v : vcycle)
         cycle.append(v);
