@@ -1,6 +1,6 @@
 // graph-tool -- a general graph modification and manipulation thingy
 //
-// Copyright (C) 2006-2023 Tiago de Paula Peixoto <tiago@skewed.de>
+// Copyright (C) 2006-2025 Tiago de Paula Peixoto <tiago@skewed.de>
 //
 // This program is free software; you can redistribute it and/or modify it under
 // the terms of the GNU Lesser General Public License as published by the Free
@@ -28,14 +28,14 @@ using namespace std;
 using namespace boost;
 using namespace graph_tool;
 
-void do_kcore_decomposition(GraphInterface& gi, boost::any prop)
+void do_kcore_decomposition(GraphInterface& gi, std::any prop)
 {
     gt_dispatch<>()
         ([](auto& g, auto core)
          {
              kcore_decomposition(g, core);
          },
-         all_graph_views(), writable_vertex_scalar_properties())
+         all_graph_views, writable_vertex_scalar_properties)
         (gi.get_graph_view(), prop);
 }
 
