@@ -1,6 +1,6 @@
 // graph-tool -- a general graph modification and manipulation thingy
 //
-// Copyright (C) 2006-2023 Tiago de Paula Peixoto <tiago@skewed.de>
+// Copyright (C) 2006-2025 Tiago de Paula Peixoto <tiago@skewed.de>
 //
 // This program is free software; you can redistribute it and/or modify it under
 // the terms of the GNU Lesser General Public License as published by the Free
@@ -172,7 +172,7 @@ struct do_maximal_vertex_set
     }
 };
 
-void maximal_vertex_set(GraphInterface& gi, boost::any mvs, bool high_deg,
+void maximal_vertex_set(GraphInterface& gi, std::any mvs, bool high_deg,
                         rng_t& rng)
 {
     run_action<>()
@@ -183,7 +183,7 @@ void maximal_vertex_set(GraphInterface& gi, boost::any mvs, bool high_deg,
                  (std::forward<decltype(graph)>(graph), gi.get_vertex_index(),
                   std::forward<decltype(a2)>(a2), high_deg, rng);
          },
-         writable_vertex_scalar_properties())(mvs);
+         writable_vertex_scalar_properties)(mvs);
 }
 
 #define __MOD__ topology
