@@ -1,6 +1,6 @@
 // graph-tool -- a general graph modification and manipulation thingy
 //
-// Copyright (C) 2006-2023 Tiago de Paula Peixoto <tiago@skewed.de>
+// Copyright (C) 2006-2025 Tiago de Paula Peixoto <tiago@skewed.de>
 //
 // This program is free software; you can redistribute it and/or modify it under
 // the terms of the GNU Lesser General Public License as published by the Free
@@ -24,18 +24,16 @@ using namespace std;
 using namespace boost;
 using namespace graph_tool;
 
-void percolate_edge(GraphInterface& gi, boost::any tree, boost::any size,
+void percolate_edge(GraphInterface& gi, std::any tree, std::any size,
                     python::object edges, python::object max_size, bool second)
 {
-    typedef property_map_type::apply<int64_t,
-                                     GraphInterface::vertex_index_map_t>::type
-        tree_t;
+    typedef vprop_map_t<int64_t> tree_t;
     tree_t tree_map;
     try
     {
-        tree_map = any_cast<tree_t>(tree);
+        tree_map = std::any_cast<tree_t>(tree);
     }
-    catch (bad_any_cast&)
+    catch (std::bad_any_cast&)
     {
         throw GraphException("tree map must be a vertex property map of value type int64_t");
     }
@@ -43,9 +41,9 @@ void percolate_edge(GraphInterface& gi, boost::any tree, boost::any size,
     tree_t size_map;
     try
     {
-        size_map = any_cast<tree_t>(size);
+        size_map = std::any_cast<tree_t>(size);
     }
-    catch (bad_any_cast&)
+    catch (std::bad_any_cast&)
     {
         throw GraphException("size map must be a vertex property map of value type int64_t");
     }
@@ -53,24 +51,22 @@ void percolate_edge(GraphInterface& gi, boost::any tree, boost::any size,
     multi_array_ref<uint64_t, 2> es = get_array<uint64_t, 2>(edges);
     multi_array_ref<uint64_t, 1> ms = get_array<uint64_t, 1>(max_size);
 
-    run_action<graph_tool::detail::never_directed>()
+    run_action<decltype(graph_tool::never_directed)>()
         (gi, [&](auto& g){ edge_percolate(g, tree_map, size_map, ms, es, second); })();
 }
 
 
-void percolate_vertex(GraphInterface& gi, boost::any tree, boost::any size,
-                      boost::any visited, python::object vertices,
+void percolate_vertex(GraphInterface& gi, std::any tree, std::any size,
+                      std::any visited, python::object vertices,
                       python::object max_size, bool second)
 {
-    typedef property_map_type::apply<int64_t,
-                                     GraphInterface::vertex_index_map_t>::type
-        tree_t;
+    typedef vprop_map_t<int64_t> tree_t;
     tree_t tree_map;
     try
     {
-        tree_map = any_cast<tree_t>(tree);
+        tree_map = std::any_cast<tree_t>(tree);
     }
-    catch (bad_any_cast&)
+    catch (std::bad_any_cast&)
     {
         throw GraphException("tree map must be a vertex property map of value type int64_t");
     }
@@ -78,23 +74,21 @@ void percolate_vertex(GraphInterface& gi, boost::any tree, boost::any size,
     tree_t size_map;
     try
     {
-        size_map = any_cast<tree_t>(size);
+        size_map = std::any_cast<tree_t>(size);
     }
-    catch (bad_any_cast&)
+    catch (std::bad_any_cast&)
     {
         throw GraphException("size map must be a vertex property map of value type int64_t");
     }
 
-    typedef property_map_type::apply<uint8_t,
-                                     GraphInterface::vertex_index_map_t>::type
-        visited_t;
+    typedef vprop_map_t<uint8_t> visited_t;
 
     visited_t visited_map;
     try
     {
-        visited_map = any_cast<visited_t>(visited);
+        visited_map = std::any_cast<visited_t>(visited);
     }
-    catch (bad_any_cast&)
+    catch (std::bad_any_cast&)
     {
         throw GraphException("visited map must be a vertex property map of value type uint8_t");
     }
@@ -102,7 +96,7 @@ void percolate_vertex(GraphInterface& gi, boost::any tree, boost::any size,
     multi_array_ref<uint64_t, 1> vs = get_array<uint64_t, 1>(vertices);
     multi_array_ref<uint64_t, 1> ms = get_array<uint64_t, 1>(max_size);
 
-    run_action<graph_tool::detail::never_directed>()
+    run_action<decltype(graph_tool::never_directed)>()
         (gi, [&](auto& g){ vertex_percolate(g, tree_map, size_map, visited_map,
                                             ms, vs, second); })();
 }
