@@ -1,6 +1,6 @@
 // graph-tool -- a general graph modification and manipulation thingy
 //
-// Copyright (C) 2006-2023 Tiago de Paula Peixoto <tiago@skewed.de>
+// Copyright (C) 2006-2025 Tiago de Paula Peixoto <tiago@skewed.de>
 //
 // This program is free software; you can redistribute it and/or modify it under
 // the terms of the GNU Lesser General Public License as published by the Free
@@ -94,14 +94,14 @@ struct do_random_matching
     }
 };
 
-void random_matching(GraphInterface& gi, boost::any weight, boost::any match,
+void random_matching(GraphInterface& gi, std::any weight, std::any match,
                      bool minimize, rng_t& rng)
 {
     typedef UnityPropertyMap<int,GraphInterface::edge_t> weight_map_t;
-    typedef mpl::push_back<edge_scalar_properties, weight_map_t>::type
-        edge_props_t;
+    auto edge_props_t = hana::append(edge_scalar_properties,
+                                     hana::type<weight_map_t>());
 
-    if(weight.empty())
+    if(!weight.has_value())
         weight = weight_map_t();
 
     run_action<>()
@@ -113,7 +113,7 @@ void random_matching(GraphInterface& gi, boost::any weight, boost::any match,
                   std::forward<decltype(a2)>(a2),
                   std::forward<decltype(a3)>(a3), minimize, rng);
          },
-         edge_props_t(), writable_vertex_scalar_properties())(weight, match);
+         edge_props_t, writable_vertex_scalar_properties)(weight, match);
 }
 
 #include <boost/python.hpp>
