@@ -1,6 +1,6 @@
 // graph-tool -- a general graph modification and manipulation thingy
 //
-// Copyright (C) 2006-2023 Tiago de Paula Peixoto <tiago@skewed.de>
+// Copyright (C) 2006-2025 Tiago de Paula Peixoto <tiago@skewed.de>
 //
 // This program is free software; you can redistribute it and/or modify it under
 // the terms of the GNU Lesser General Public License as published by the Free
@@ -28,51 +28,49 @@ using namespace std;
 using namespace boost;
 using namespace graph_tool;
 
-template <class Type, class Index>
-auto uncheck(boost::unchecked_vector_property_map<Type,Index>, boost::any p)
-{
-    return boost::any_cast<boost::checked_vector_property_map<Type,Index>>(p).get_unchecked();
-}
-
-template <class T>
-auto&& uncheck(T&&, boost::any p)
-{
-    return boost::any_cast<T>(p);
-}
-
-typedef UnityPropertyMap<size_t,GraphInterface::edge_t> ecmap_t;
-typedef boost::mpl::push_back<edge_scalar_properties, ecmap_t>::type
-        weight_props_t;
-
 python::object similarity(GraphInterface& gi1, GraphInterface& gi2,
-                          boost::any weight1, boost::any weight2,
-                          boost::any label1, boost::any label2, double norm,
+                          std::any weight1, std::any weight2,
+                          std::any label1, std::any label2, double norm,
                           bool asym)
 {
-    if (weight1.empty())
+    typedef UnityPropertyMap<size_t,GraphInterface::edge_t> ecmap_t;
+    auto weight_props_t = hana::tuple_t<ecmap_t, eprop_map_t<long double>,
+                                        edprop_map_t<long double>>;
+
+    if (!weight1.has_value())
         weight1 = ecmap_t();
-    if (weight2.empty())
+    else
+        weight1 = eprop_map_as_dynamic(weight1, eprop_map_t<long double>());
+
+    if (!weight2.has_value())
         weight2 = ecmap_t();
+    else
+        weight2 = eprop_map_as_dynamic(weight2, eprop_map_t<long double>());
+
     python::object s;
-    gt_dispatch<>()
-        ([&](const auto& g1, const auto& g2, auto ew1, auto l1)
+
+    gt_dispatch<>(false)
+        ([&](const auto& g1, const auto& g2, auto ew1, auto ew2, auto l1, auto l2)
          {
-             auto l2 = uncheck(l1, label2);
-             auto ew2 = uncheck(ew1, weight2);
+             GILRelease gil_release;
              auto ret = get_similarity(g1, g2, ew1, ew2, l1, l2, norm, asym);
+             gil_release.restore();
              s = python::object(ret);
          },
-         all_graph_views(),
-         all_graph_views(),
-         weight_props_t(),
-         vertex_scalar_properties())
-        (gi1.get_graph_view(), gi2.get_graph_view(), weight1, label1);
+         all_graph_views,
+         all_graph_views,
+         weight_props_t,
+         same_arg_type,
+         vertex_scalar_properties,
+         same_arg_type)
+        (gi1.get_graph_view(), gi2.get_graph_view(), weight1, weight2,
+         label1, label2);
     return s;
 }
 
 python::object similarity_fast(GraphInterface& gi1, GraphInterface& gi2,
-                               boost::any weight1, boost::any weight2,
-                               boost::any label1, boost::any label2,
+                               std::any weight1, std::any weight2,
+                               std::any label1, std::any label2,
                                double norm, bool asym);
 
 #define __MOD__ topology
