@@ -1,6 +1,6 @@
 // graph-tool -- a general graph modification and manipulation thingy
 //
-// Copyright (C) 2006-2023 Tiago de Paula Peixoto <tiago@skewed.de>
+// Copyright (C) 2006-2025 Tiago de Paula Peixoto <tiago@skewed.de>
 //
 // This program is free software; you can redistribute it and/or modify it under
 // the terms of the GNU Lesser General Public License as published by the Free
@@ -23,9 +23,9 @@
 #include <boost/graph/vf2_sub_graph_iso.hpp>
 #include <graph_python_interface.hh>
 
-using namespace graph_tool;
 using namespace boost;
 using namespace std;
+using namespace graph_tool;
 
 struct ListMatch
 {
@@ -103,7 +103,7 @@ struct GenMatch
                     return true;
                 vmap[v] = w;
             }
-            _yield(boost::python::object(PythonPropertyMap<VertexMap>(c_vmap)));
+            _yield(boost::python::object(graph_tool::PythonPropertyMap<VertexMap>(c_vmap)));
             return true;
         }
 
@@ -131,13 +131,13 @@ struct get_subgraphs
     template <class Graph1, class Graph2, class VertexLabel,
               class EdgeLabel, class VertexMap, class Matcher>
     void operator()(const Graph1& sub, const Graph2& g,
-                    VertexLabel vertex_label1, boost::any avertex_label2,
-                    EdgeLabel edge_label1, boost::any aedge_label2,
+                    VertexLabel vertex_label1, std::any avertex_label2,
+                    EdgeLabel edge_label1, std::any aedge_label2,
                     vector<VertexMap>& vmaps, size_t max_n, bool induced,
                     bool iso, Matcher m) const
     {
-        VertexLabel vertex_label2 = any_cast<VertexLabel>(avertex_label2);
-        EdgeLabel edge_label2 = any_cast<EdgeLabel>(aedge_label2);
+        VertexLabel vertex_label2 = std::any_cast<VertexLabel>(avertex_label2);
+        EdgeLabel edge_label2 = std::any_cast<EdgeLabel>(aedge_label2);
 
         auto matcher = m.get_match(sub, g, vmaps,max_n);
 
@@ -176,52 +176,44 @@ struct get_subgraphs
 
 boost::python::object
 subgraph_isomorphism(GraphInterface& gi1, GraphInterface& gi2,
-                     boost::any vertex_label1, boost::any vertex_label2,
-                     boost::any edge_label1, boost::any edge_label2,
+                     std::any vertex_label1, std::any vertex_label2,
+                     std::any edge_label1, std::any edge_label2,
                      size_t max_n, bool induced, bool iso, bool generator)
 {
-    // typedef mpl::push_back<vertex_properties,
-    //                        UnityPropertyMap<bool,GraphInterface::vertex_t> >
-    //     ::type vertex_props_t;
+    typedef vprop_map_t<int64_t> vlabel_t;
+    auto vertex_props = hana::tuple_t<typename vlabel_t::unchecked_t,
+                                      UnityPropertyMap<bool,
+                                                       GraphInterface::vertex_t> >;
 
-    // typedef mpl::push_back<edge_properties,
-    //                        UnityPropertyMap<bool,GraphInterface::edge_t> >
-    //     ::type edge_props_t;
-
-    typedef vprop_map_t<int64_t>::type vlabel_t;
-    typedef mpl::vector2<typename vlabel_t::unchecked_t,
-                         UnityPropertyMap<bool,
-                                          GraphInterface::vertex_t> > vertex_props_t;
-
-    typedef eprop_map_t<int64_t>::type elabel_t;
-    typedef mpl::vector2<typename elabel_t::unchecked_t,
-                         UnityPropertyMap<bool,
-                                          GraphInterface::edge_t> > edge_props_t;
+    typedef eprop_map_t<int64_t> elabel_t;
+    auto edge_props = hana::tuple_t<typename elabel_t::unchecked_t,
+                                    UnityPropertyMap<bool,
+                                                     GraphInterface::edge_t> >;
 
 
     if (gi1.get_directed() != gi2.get_directed())
         return boost::python::object();
 
-    if (vertex_label1.empty() || vertex_label2.empty())
+    if (!vertex_label1.has_value() || !vertex_label2.has_value())
     {
         vertex_label1 = vertex_label2 =
             UnityPropertyMap<bool,GraphInterface::vertex_t>();
     }
     else
     {
-        vertex_label1 = any_cast<vlabel_t>(vertex_label1).get_unchecked(num_vertices(gi1.get_graph()));
-        vertex_label2 = any_cast<vlabel_t>(vertex_label2).get_unchecked(num_vertices(gi2.get_graph()));
+        vertex_label1 = std::any_cast<vlabel_t>(vertex_label1).get_unchecked(num_vertices(gi1.get_graph()));
+        vertex_label2 = std::any_cast<vlabel_t>(vertex_label2).get_unchecked(num_vertices(gi2.get_graph()));
     }
 
-    if (edge_label1.empty() || edge_label2.empty())
+    if (!edge_label1.has_value() || !edge_label2.has_value())
     {
         edge_label1 = edge_label2 =
             UnityPropertyMap<bool,GraphInterface::edge_t>();
     }
     else
     {
-        edge_label1 = any_cast<elabel_t>(edge_label1).get_unchecked(gi1.get_edge_index_range());
-        edge_label2 = any_cast<elabel_t>(edge_label2).get_unchecked(gi2.get_edge_index_range());
+        edge_label1 = std::any_cast<elabel_t>(edge_label1).get_unchecked(gi1.get_edge_index_range());
+        edge_label2 = std::any_cast<elabel_t>(edge_label2).get_unchecked(gi2.get_edge_index_range());
     }
 
     vector<vlabel_t> vmaps;
@@ -230,39 +222,39 @@ subgraph_isomorphism(GraphInterface& gi1, GraphInterface& gi2,
         gt_dispatch<>()
             ([&](auto&& graph, auto&& a2, auto&& a3, auto&& a4)
              {
-                 return get_subgraphs()
+                 get_subgraphs()
                      (std::forward<decltype(graph)>(graph),
                       std::forward<decltype(a2)>(a2),
                       std::forward<decltype(a3)>(a3), vertex_label2,
                       std::forward<decltype(a4)>(a4), edge_label2, vmaps,
                       max_n, induced, iso, ListMatch());
              },
-             all_graph_views(), all_graph_views(), vertex_props_t(),
-             edge_props_t())(gi1.get_graph_view(), gi2.get_graph_view(),
+             all_graph_views, all_graph_views, vertex_props,
+             edge_props)(gi1.get_graph_view(), gi2.get_graph_view(),
                              vertex_label1, edge_label1);
 
         python::list vmapping;
         for (auto& vmap: vmaps)
             vmapping.append(PythonPropertyMap<vlabel_t>(vmap));
-        return std::move(vmapping);
+        return vmapping;
     }
     else
     {
 #ifdef HAVE_BOOST_COROUTINE
         auto dispatch = [&](auto& yield)
         {
-            run_action<>()
+            run_action<>(false)
                 (gi1,
                  [&](auto&& graph, auto&& a2, auto&& a3, auto&& a4)
                  {
-                     return get_subgraphs()
+                     get_subgraphs()
                          (std::forward<decltype(graph)>(graph),
                           std::forward<decltype(a2)>(a2),
                           std::forward<decltype(a3)>(a3), vertex_label2,
                           std::forward<decltype(a4)>(a4), edge_label2, vmaps,
                           max_n, induced, iso, GenMatch(yield));
                  },
-                 all_graph_views(), vertex_props_t(), edge_props_t())
+                 all_graph_views, vertex_props, edge_props)
                 (gi2.get_graph_view(), vertex_label1, edge_label1);
         };
         CoroGenerator gen(dispatch);
