@@ -2,7 +2,7 @@
 # -*- coding: utf-8 -*-
 #
 #
-# Copyright (C) 2006-2023 Tiago de Paula Peixoto <tiago@skewed.de>
+# Copyright (C) 2006-2025 Tiago de Paula Peixoto <tiago@skewed.de>
 #
 # This program is free software; you can redistribute it and/or modify it under
 # the terms of the GNU Lesser General Public License as published by the Free
@@ -107,3 +107,22 @@ def _copy_func(f, name=None):
                             f.__defaults__, f.__closure__)
     fn.__dict__.update(f.__dict__)
     return fn
+
+def _parallel(f):
+    text = """.. admonition:: Parallel implementation.
+
+    If enabled during compilation, this algorithm will run in parallel using
+    `OpenMP <https://en.wikipedia.org/wiki/OpenMP>`_. See the :ref:`parallel
+    algorithms <parallel_algorithms>` section for information about how to
+    control several aspects of parallelization.
+    """
+    lines = f.__doc__.split("\n")
+    f.__doc__ = ""
+    for line in lines:
+        if "@parallel@" in line:
+            indent = line.replace("@parallel@", "")
+            for l in text.split("\n"):
+                f.__doc__ += indent + l + "\n"
+        else:
+            f.__doc__ += line + "\n"
+    return f
