@@ -3,7 +3,7 @@
 #
 # graph_tool -- a general graph manipulation python module
 #
-# Copyright (C) 2006-2023 Tiago de Paula Peixoto <tiago@skewed.de>
+# Copyright (C) 2006-2025 Tiago de Paula Peixoto <tiago@skewed.de>
 #
 # This program is free software; you can redistribute it and/or modify it under
 # the terms of the GNU Lesser General Public License as published by the Free
@@ -70,17 +70,25 @@ Low-level graph drawing
    GraphWidget
    GraphWindow
 
+.. autosummary::
+   :nosignatures:
+   :toctree: autosummary
+
+   GraphArtist
+
 """
 
 from .. import Graph, GraphView, _check_prop_vector, _check_prop_scalar, \
     group_vector_property, ungroup_vector_property, infect_vertex_property, \
-    _prop, _get_rng, VertexPropertyMap
+    _prop, _get_rng, VertexPropertyMap, perfect_prop_hash
 from .. topology import max_cardinality_matching, max_independent_vertex_set, \
     label_components, shortest_distance, make_maximal_planar, is_planar
 from .. generation import predecessor_tree, condensation_graph
+from .. inference import minimize_blockmodel_dl, BlockState, ModularityState
 from .. inference.util import nested_contiguous_map
 import numpy.random
 from numpy import sqrt
+from collections.abc import Iterable
 
 from .. dl_import import dl_import
 dl_import("from . import libgraph_tool_layout")
@@ -324,7 +332,7 @@ def fruchterman_reingold_layout(g, weight=None, a=None, r=1., scale=None,
     return pos
 
 
-def arf_layout(g, weight=None, d=0.5, a=10, dt=0.001, epsilon=1e-6,
+def arf_layout(g, weight=None, d=.5, a=10, dt=0.001, epsilon=1e-6,
                max_iter=1000, pos=None, dim=2):
     r"""Calculate the ARF spring-block layout of the graph.
 
@@ -334,9 +342,9 @@ def arf_layout(g, weight=None, d=0.5, a=10, dt=0.001, epsilon=1e-6,
         Graph to be used.
     weight : :class:`~graph_tool.EdgePropertyMap` (optional, default: ``None``)
         An edge property map with the respective weights.
-    d : float (optional, default: ``0.5``)
+    d : float (optional, default: ``1``)
         Opposing force between vertices.
-    a : float (optional, default: ``10``)
+    a : float (optional, default: ``1.1``)
         Attracting force between adjacent vertices.
     dt : float (optional, default: ``0.001``)
         Iteration step size.
@@ -366,11 +374,11 @@ def arf_layout(g, weight=None, d=0.5, a=10, dt=0.001, epsilon=1e-6,
     .. testcode::
        :hide:
 
-       np.random.seed(43)
-       gt.seed_rng(45)
+       np.random.seed(42)
+       gt.seed_rng(42)
 
     >>> g = gt.price_network(300)
-    >>> pos = gt.arf_layout(g)
+    >>> pos = gt.arf_layout(g, a=5)
     >>> gt.graph_draw(g, pos=pos, output="graph-draw-arf.pdf")
     <...>
 
@@ -404,9 +412,10 @@ def arf_layout(g, weight=None, d=0.5, a=10, dt=0.001, epsilon=1e-6,
     return pos
 
 
-def _coarse_graph(g, vweight, eweight, mivs=False, groups=None):
+def _coarse_graph(g, vweight, eweight, kind="matching", groups=None):
+    mivs = None
     if groups is None:
-        if mivs:
+        if kind == "mivs":
             mivs = max_independent_vertex_set(g, high_deg=True)
             u = GraphView(g, vfilt=mivs, directed=False)
             c = label_components(u)[0]
@@ -415,8 +424,7 @@ def _coarse_graph(g, vweight, eweight, mivs=False, groups=None):
             infect_vertex_property(u, c,
                                    list(range(1, c.fa.max() + 1)))
             c = g.own_property(c)
-        else:
-            mivs = None
+        elif kind == "ec":
             m = max_cardinality_matching(GraphView(g, directed=False),
                                          heuristic=True, weight=eweight,
                                          minimize=False, edges=True)
@@ -424,11 +432,31 @@ def _coarse_graph(g, vweight, eweight, mivs=False, groups=None):
             c = label_components(u)[0]
             c = g.own_property(c)
             u = GraphView(g, directed=False)
+        elif kind == "modularity":
+            B = max(g.num_vertices() // 2, 1)
+            state = minimize_blockmodel_dl(GraphView(g, directed=False),
+                                           state=ModularityState,
+                                           state_args=dict(eweight=eweight),
+                                           multilevel_mcmc_args=dict(B_min=B,
+                                                                     B_max=B))
+            c = state.b.copy()
+        elif kind in ["sbm", "dcsbm"]:
+            B = max(g.num_vertices() // 2, 1)
+            w = eweight.copy("int") if eweight is not None else None
+            state = minimize_blockmodel_dl(GraphView(g, directed=False),
+                                           state=BlockState,
+                                           state_args=dict(eweight=w,
+                                                           deg_corr=kind == "dcsbm"),
+                                           multilevel_mcmc_args=dict(B_min=B,
+                                                                     B_max=B))
+            c = state.b.copy()
+        else:
+            raise ValueError("invalid coarsening method:", kind)
     else:
-        mivs = None
         c = g.new_vp("int", vals=groups)
-    cg, cc, vcount, ecount = condensation_graph(g, c, vweight, eweight)[:4]
-    return cg, cc, vcount, ecount, c, mivs
+    c = perfect_prop_hash([c])[0]
+    cg, vcount, ecount = condensation_graph(g, c, vweight, eweight)
+    return cg, cg.vertex_index.copy("int32_t"), vcount, ecount, c, mivs
 
 
 def _propagate_pos(g, cg, c, cc, cpos, delta, mivs):
@@ -448,13 +476,10 @@ def _propagate_pos(g, cg, c, cc, cpos, delta, mivs):
     if mivs is not None:
         g = g.base
         u = GraphView(g, directed=False)
-        try:
-            libgraph_tool_layout.propagate_pos_mivs(u._Graph__graph,
-                                                    _prop("v", u, mivs),
-                                                    _prop("v", u, pos),
-                                                    delta, _get_rng())
-        except ValueError:
-            graph_draw(u, mivs, vertex_fillcolor=mivs)
+        libgraph_tool_layout.propagate_pos_mivs(u._Graph__graph,
+                                                _prop("v", u, mivs),
+                                                _prop("v", u, pos),
+                                                delta, _get_rng())
     return pos
 
 
@@ -468,30 +493,34 @@ def _avg_edge_distance(g, pos):
 
 def coarse_graphs(g, method="hybrid", mivs_thres=0.9, ec_thres=0.75,
                   weighted_coarse=False, eweight=None, vweight=None,
-                  groups=None, verbose=False):
+                  groups=None, gamma=None, verbose=False):
     cg = [[g, None, None, None, None, None]]
     if weighted_coarse:
         cg[-1][2], cg[-1][3] = vweight, eweight
-    mivs = not (method in ["hybrid", "ec"])
+    hybrid = False
+    if method == "hybrid":
+        method = "ec"
+        hybrid = True
+    if verbose:
+        print(f"Coarse level (none): 0, N={g.num_vertices()}")
     while True:
         if groups is None or len(groups) < len(cg):
             b = None
         else:
             b = groups[len(cg)-1]
-        u = _coarse_graph(cg[-1][0], cg[-1][2], cg[-1][3], mivs, b)
-        thres = mivs_thres if mivs else ec_thres
-        if u[0].num_vertices() >= thres * cg[-1][0].num_vertices():
-            if method == "hybrid" and not mivs:
-                mivs = True
-            else:
-                break
+        u = _coarse_graph(cg[-1][0], cg[-1][2], cg[-1][3], method, b)
+        if hybrid:
+            thres = mivs_thres if method == "mivs" else ec_thres
+            if u[0].num_vertices() >= thres * cg[-1][0].num_vertices():
+                if method == "ec":
+                    method = "mivs"
+                else:
+                    break
         if u[0].num_vertices() <= 2:
             break
         cg.append(u)
         if verbose:
-            print("Coarse level (%s):" % ("MIVS" if mivs else "EC"), end=' ')
-            print(len(cg), " num vertices:", end=' ')
-            print(u[0].num_vertices())
+            print(f"Coarse level ({method}): {len(cg)}, N={u[0].num_vertices()}")
     cg.reverse()
     Ks = []
     pos = random_layout(cg[0][0], dim=2)
@@ -504,10 +533,17 @@ def coarse_graphs(g, method="hybrid", mivs_thres=0.9, ec_thres=0.75,
             Ks.append(K)
             continue
         if weighted_coarse:
-            gamma = 1.
+            r = 1.
         else:
-            gamma = 0.75
-        Ks.append(Ks[-1] * gamma)
+            r = 0.75
+        Ks.append(Ks[-1] * r)
+
+    if gamma is None:
+        gamma = [0] * len(cg)
+    else:
+        if not isinstance(gamma, Iterable):
+            gamma = [gamma]
+        gamma = list(gamma) + [0] * (len(cg) - len(gamma))
 
     for i in range(len(cg)):
         u, cc, vcount, ecount, c, mivs = cg[i]
@@ -515,7 +551,7 @@ def coarse_graphs(g, method="hybrid", mivs_thres=0.9, ec_thres=0.75,
             b = None
         else:
             b = groups[len(cg) - 1 - i:]
-        yield u, pos, Ks[i], vcount, ecount, b
+        yield u, pos, Ks[i], vcount, ecount, b, gamma[len(cg)-i-1:]
 
         if verbose:
             print("avg edge distance:", _avg_edge_distance(u, pos))
@@ -525,15 +561,18 @@ def coarse_graphs(g, method="hybrid", mivs_thres=0.9, ec_thres=0.75,
                 print("propagating...", end=' ')
                 print(mivs.a.sum() if mivs is not None else "")
             pos = _propagate_pos(cg[i + 1][0], u, c, cc, pos,
-                                 Ks[i] / 1000., mivs)
+                                 #Ks[i] / 1000.,
+                                 1e-6,
+                                 mivs)
 
 
-def sfdp_layout(g, vweight=None, eweight=None, pin=None, groups=None, C=0.2,
-                K=None, p=2., theta=0.6, max_level=15, r=1., gamma=.3, mu=2.,
-                kappa=1., rmap=None, R=1, init_step=None, cooling_step=0.95,
-                adaptive_cooling=True, epsilon=1e-2, max_iter=0, pos=None,
-                multilevel=None, coarse_method="hybrid", mivs_thres=0.9,
-                ec_thres=0.75, weighted_coarse=False, verbose=False):
+def sfdp_layout(g, vweight=None, eweight=None, pin=None, C=0.2, K=None, p=2.,
+                theta=0.6, max_level=15, r=1., kc=10, groups=None, gamma=.1,
+                mu=2., kappa=1., rmap=None, R=1, init_step=None,
+                cooling_step=0.95, adaptive_cooling=True, epsilon=1e-2,
+                max_iter=0, pos=None, multilevel=None, coarse_method="hybrid",
+                mivs_thres=0.9, ec_thres=0.75, weighted_coarse=False,
+                verbose=False):
     r"""Obtain the SFDP spring-block layout of the graph.
 
     Parameters
@@ -547,9 +586,6 @@ def sfdp_layout(g, vweight=None, eweight=None, pin=None, groups=None, C=0.2,
     pin : :class:`~graph_tool.VertexPropertyMap` (optional, default: ``None``)
         A vertex property map with boolean values, which, if given,
         specify the vertices which will not have their positions modified.
-    groups : :class:`~graph_tool.VertexPropertyMap` (optional, default: ``None``)
-        A vertex property map with group assignments. Vertices belonging to the
-        same group will be put close together.
     C : float (optional, default: ``0.2``)
         Relative strength of repulsive forces.
     K : float (optional, default: ``None``)
@@ -563,12 +599,18 @@ def sfdp_layout(g, vweight=None, eweight=None, pin=None, groups=None, C=0.2,
         Maximum quadtree level.
     r : float (optional, default: ``1.``)
         Strength of attractive force between connected components.
-    gamma : float (optional, default: ``.3``)
-        Strength of the repulsive force between different groups.
-    mu : float (optional, default: ``2``)
-        Typical length of the repulsive force between different groups.
-    kappa : float (optional, default: ``1.0``)
-        Multiplicative factor on the attracttive force between nodes of the same group.
+    kc : int (optional, default: ``10``)
+        Number of connected components to subsample.
+    groups : :class:`~graph_tool.VertexPropertyMap` or list of list of integers (optional, default: ``None``)
+        A vertex property map with group assignments. Vertices belonging to the
+        same group will be put close together. Optionally, this can take an
+        hierarchical partition, represented by a list of lists, where the list
+        above correspond to the partition of the list below.
+    gamma : float or list of floats (optional, default: ``.1``)
+        Strength of the attractive force between nodes of the same group to
+        their center of mass. In case of a hierarchical partition, this should
+        correspond to a list of floats, containing the strenghts for each
+        hierarchical level. This option has no effect if ``groups`` is ``None``.
     rmap : :class:`~graph_tool.VertexPropertyMap` (optional, default: ``None``)
         Vertex rank to be used around to order them preferentially in the
         :math:`y` direction.
@@ -651,10 +693,20 @@ def sfdp_layout(g, vweight=None, eweight=None, pin=None, groups=None, C=0.2,
     _check_prop_vector(pos, name="pos", floating=True)
 
     if isinstance(groups, VertexPropertyMap):
-        groups = [numpy.asarray(groups.a, dtype="int32")]
+        groups = [numpy.asarray(groups.a.copy(), dtype="int32")]
     elif groups is not None:
+        if isinstance(groups[0], VertexPropertyMap):
+            groups = [groups[0].a.copy()] + list(groups[1:])
         groups = nested_contiguous_map(groups)
 
+    if groups is None:
+        gamma = [0]
+    else:
+        if not isinstance(gamma, Iterable):
+            gamma = [gamma]
+        if len(gamma) < len(groups):
+            gamma = list(gamma) + [0] * (len(groups) - len(gamma))
+
     g_ = g
     g = GraphView(g, directed=False)
 
@@ -691,18 +743,19 @@ def sfdp_layout(g, vweight=None, eweight=None, pin=None, groups=None, C=0.2,
                             eweight=eweight,
                             vweight=vweight,
                             groups=groups,
+                            gamma=gamma,
                             verbose=verbose)
-        for count, (u, pos, K, vcount, ecount, groups) in enumerate(cgs):
+        for count, (u, pos, K_, vcount, ecount, groups, gamma) in enumerate(cgs):
             if verbose:
-                print("Positioning level:", count, u.num_vertices(), end=' ')
-                print("with K =", K, "...")
-                count += 1
+                print(f"Positioning level: {count}, {u.num_vertices()}", end=" ")
+                print(f"with K={K_}, gamma={gamma} ...")
             pos = sfdp_layout(u, pos=pos,
                               vweight=vcount if weighted_coarse else None,
                               eweight=ecount if weighted_coarse else None,
                               groups=groups,
-                              C=C, K=K, p=p,
-                              theta=theta, gamma=gamma, mu=mu, kappa=kappa,
+                              C=C, K=K_, p=p,
+                              theta=theta,
+                              gamma=gamma,
                               epsilon=epsilon,
                               max_iter=max_iter,
                               cooling_step=cooling_step,
@@ -734,7 +787,7 @@ def sfdp_layout(g, vweight=None, eweight=None, pin=None, groups=None, C=0.2,
                                      _prop("v", g, vweight),
                                      _prop("e", g, eweight),
                                      _prop("v", g, pin),
-                                     (C, K, p, gamma, mu, kappa, groups, r,
+                                     (C, K, p, gamma, groups, r, kc,
                                       _prop("v", g, c), R, _prop("v", g, rmap)),
                                      theta, init_step, cooling_step, max_level,
                                      epsilon, max_iter, not adaptive_cooling,
@@ -861,8 +914,8 @@ except ImportError:
 
 try:
     from . cairo_draw import GraphWidget, GraphWindow, \
-        interactive_window
-    __all__ += ["interactive_window", "GraphWidget", "GraphWindow"]
+        interactive_window, GraphArtist
+    __all__ += ["interactive_window", "GraphWidget", "GraphWindow", "GraphArtist"]
 except ImportError:
     pass
 
