@@ -3,7 +3,7 @@
 #
 # graph_tool -- a general graph manipulation python module
 #
-# Copyright (C) 2006-2023 Tiago de Paula Peixoto <tiago@skewed.de>
+# Copyright (C) 2006-2025 Tiago de Paula Peixoto <tiago@skewed.de>
 #
 # This program is free software; you can redistribute it and/or modify it under
 # the terms of the GNU Lesser General Public License as published by the Free
@@ -80,7 +80,7 @@ def modularity(g, b, gamma=1., weight=None):
        :doi:`10.1073/pnas.0601602103`, :arxiv:`physics/0602124`
     """
 
-    if b.value_type() not in ["bool", "int16_t", "int32_t", "int64_t",
+    if b.value_type() not in ["bool", "int16_t", "int64_t", "int64_t",
                               "unsigned long"]:
         b = perfect_prop_hash([b])[0]
     Q = libinference.modularity(g._Graph__graph, gamma,
@@ -89,6 +89,7 @@ def modularity(g, b, gamma=1., weight=None):
     return Q
 
 
+@entropy_state_signature
 class ModularityState(MCMCState, MultiflipMCMCState, MultilevelMCMCState,
                       GibbsMCMCState, DrawBlockState):
     r"""Obtain the partition of a network according to the maximization of Newman's modularity.
@@ -103,7 +104,7 @@ class ModularityState(MCMCState, MultiflipMCMCState, MultilevelMCMCState,
        For a more detailed explanation see `“Modularity maximization
        considered harmful”
        <https://skewed.de/tiago/blog/modularity-harmful>`_, as well as
-       [peixoto-descriptive-2021]_.
+       [peixoto-descriptive-2023]_.
 
        Do not use this approach in the analysis of networks without
        understanding the consequences. This algorithm is included only for
@@ -122,36 +123,39 @@ class ModularityState(MCMCState, MultiflipMCMCState, MultilevelMCMCState,
         be used.
     eweight : :class:`~graph_tool.EdgePropertyMap` (optional, default: ``None``)
         Edge multiplicities (for multigraphs).
+    entropy_args: ``dict`` (optional, default: ``{}``)
+        Override default arguments for :meth:`~ModularityState.entropy()`
+        method and releated operations.
 
+    References
+    ----------
+    .. [peixoto-descriptive-2023] Tiago P. Peixoto, "Descriptive vs.
+       inferential community detection in networks: pitfalls, myths and
+       half-truths", Elements in the Structure and Dynamics of Complex Networks,
+       Cambridge University Press (2023), :doi:`10.1017/9781009118897`
+       :arxiv:`2112.00183`
     """
 
-    def __init__(self, g, b=None, eweight=None):
-
+    def __init__(self, g, b=None, eweight=None, entropy_args={}):
+        EntropyState.__init__(self, entropy_args=entropy_args)
         self.g = GraphView(g, directed=False)
         if b is None:
-            self.b = self.g.new_vp("int32_t")
+            self.b = self.g.new_vp("int64_t")
         elif isinstance(b, PropertyMap):
-            self.b = self.g.own_property(b).copy("int32_t")
+            self.b = self.g.own_property(b).copy("int64_t")
         else:
-            self.b = self.g.new_vp("int32_t", vals=b)
+            self.b = self.g.new_vp("int64_t", vals=b)
 
         if eweight is None:
-            eweight = g.new_ep("int", 1)
-        elif eweight.value_type() != "int32_t":
-            eweight = g.own_property(eweight.copy(value_type="int32_t"))
+            eweight = g.new_ep("int64_t", 1)
         else:
             eweight = g.own_property(eweight)
         self.eweight = eweight
 
-        self.er = Vector_size_t()
-        self.err = Vector_size_t()
-
-        self.bg = self.g
+        self.bg = Graph(g.num_vertices(), directed=False)
         self._abg = self.bg._get_any()
         self._state = libinference.make_modularity_state(self)
 
-        self._entropy_args = dict(gamma=1.)
-
     def __copy__(self):
         return self.copy()
 
@@ -162,7 +166,8 @@ class ModularityState(MCMCState, MultiflipMCMCState, MultilevelMCMCState,
                                b=b if b is not None else self.b)
 
     def __getstate__(self):
-        return dict(g=self.g, b=self.b)
+        state = EntropyState.__getstate__(self)
+        return dict(state, g=self.g, b=self.b)
 
     def __setstate__(self, state):
         self.__init__(**state)
@@ -171,6 +176,9 @@ class ModularityState(MCMCState, MultiflipMCMCState, MultilevelMCMCState,
         return "<ModularityState object with %d blocks, for graph %s, at 0x%x>" % \
             (self.get_B(), str(self.g), id(self))
 
+    def _gen_eargs(self, args):
+        return libinference.modularity_entropy_args()
+
     def get_B(self):
         r"Returns the total number of blocks."
         rs = np.unique(self.b.fa)
@@ -185,10 +193,10 @@ class ModularityState(MCMCState, MultiflipMCMCState, MultilevelMCMCState,
         w = np.array(w, dtype="double")
         w = w[w>0]
         w /= w.sum()
-        return np.exp(-(w*log(w)).sum())
+        return np.exp(-(w*np.log(w)).sum())
 
     @copy_state_wrap
-    def entropy(self, gamma=1., **kwargs):
+    def _entropy(self, gamma=1.):
         r"""Return the unnormalized negative generalized modularity.
 
         Notes
@@ -207,10 +215,6 @@ class ModularityState(MCMCState, MultiflipMCMCState, MultilevelMCMCState,
 
         eargs = self._get_entropy_args(locals(), ignore=["self", "kwargs"])
 
-        if len(kwargs) > 0:
-            raise ValueError("unrecognized keyword arguments: " +
-                             str(list(kwargs.keys())))
-
         return self._state.entropy(eargs)
 
     def _get_entropy_args(self, kwargs, ignore=None):
