@@ -1,7 +1,7 @@
 /*
  * PCG Random Number Generation for C++
  *
- * Copyright 2014-2017 Melissa O'Neill <oneill@pcg-random.org>,
+ * Copyright 2014-2021 Melissa O'Neill <oneill@pcg-random.org>,
  *                     and the PCG Project contributors.
  *
  * SPDX-License-Identifier: (Apache-2.0 OR MIT)
@@ -77,6 +77,10 @@
     #endif
 #endif
 
+#if INTPTR_MAX == INT64_MAX && !defined(PCG_64BIT_SPECIALIZATIONS)
+    #define PCG_64BIT_SPECIALIZATIONS 1
+#endif
+
 namespace pcg_extras {
 
 // Recent versions of GCC have intrinsics we can use to quickly calculate
@@ -142,14 +146,14 @@ inline bitcount_t flog2(uint32_t v)
 {
     unsigned long i;
     _BitScanReverse(&i, v);
-    return i;
+    return bitcount_t(i);
 }
 
 inline bitcount_t trailingzeros(uint32_t v)
 {
     unsigned long i;
     _BitScanForward(&i, v);
-    return i;
+    return bitcount_t(i);
 }
 
 inline bitcount_t flog2(uint64_t v)
@@ -157,7 +161,7 @@ inline bitcount_t flog2(uint64_t v)
 #if defined(_M_X64) || defined(_M_ARM) || defined(_M_ARM64)
     unsigned long i;
     _BitScanReverse64(&i, v);
-    return i;
+    return bitcount_t(i);
 #else
     // 32-bit x86
     uint32_t high = v >> 32;
@@ -171,7 +175,7 @@ inline bitcount_t trailingzeros(uint64_t v)
 #if defined(_M_X64) || defined(_M_ARM) || defined(_M_ARM64)
     unsigned long i;
     _BitScanForward64(&i, v);
-    return i;
+    return bitcount_t(i);
 #else
     // 32-bit x86
     uint32_t high = v >> 32;
@@ -230,6 +234,45 @@ inline bitcount_t trailingzeros(uint64_t v)
 
 #endif
 
+inline bitcount_t flog2(uint8_t v)
+{
+    return flog2(uint32_t(v));
+}
+
+inline bitcount_t flog2(uint16_t v)
+{
+    return flog2(uint32_t(v));
+}
+
+#if __SIZEOF_INT128__
+inline bitcount_t flog2(__uint128_t v)
+{
+    uint64_t high = uint64_t(v >> 64);
+    uint64_t low  = uint64_t(v);
+
+    return high ? 64+flog2(high) : flog2(low);
+}
+#endif
+
+inline bitcount_t trailingzeros(uint8_t v)
+{
+    return trailingzeros(uint32_t(v));
+}
+
+inline bitcount_t trailingzeros(uint16_t v)
+{
+    return trailingzeros(uint32_t(v));
+}
+
+#if __SIZEOF_INT128__
+inline bitcount_t trailingzeros(__uint128_t v)
+{
+    uint64_t high = uint64_t(v >> 64);
+    uint64_t low  = uint64_t(v);
+    return low ? trailingzeros(low) : trailingzeros(high)+64;
+}
+#endif
+
 template <typename UInt>
 inline bitcount_t clog2(UInt v)
 {
@@ -258,6 +301,7 @@ inline UInt subwithcarry(UInt x, UInt y, bool carryin, bool* carryout)
 template <typename UInt, typename UIntX2>
 class uint_x4 {
 // private:
+    static constexpr unsigned int UINT_BITS = sizeof(UInt) * CHAR_BIT;
 public:
     union {
 #if PCG_LITTLE_ENDIAN
@@ -278,7 +322,6 @@ public:
         // For the array access versions, the code that uses the array
         // must handle endian itself.  Yuck.
         UInt wa[4];
-        UIntX2 da[2];
     };
 
 public:
@@ -304,59 +347,42 @@ public:
         // Nothing (else) to do
     }
 
+    constexpr uint_x4(UIntX2 v01)
+#if PCG_LITTLE_ENDIAN
+       : d{v01, UIntX2(0)}
+#else
+       : d{UIntX2(0),v01}
+#endif
+    {
+        // Nothing (else) to do
+    }
+
     template<class Integral,
              typename std::enable_if<(std::is_integral<Integral>::value
                                       && sizeof(Integral) <= sizeof(UIntX2))
                                     >::type* = nullptr>
     constexpr uint_x4(Integral v01)
 #if PCG_LITTLE_ENDIAN
-       : d{UIntX2(v01),0UL}
+       : d{UIntX2(v01), UIntX2(0)}
 #else
-       : d{0UL,UIntX2(v01)}
+       : d{UIntX2(0), UIntX2(v01)}
 #endif
     {
         // Nothing (else) to do
     }
 
-    explicit constexpr operator uint64_t() const
+    explicit constexpr operator UIntX2() const
     {
         return d.v01;
     }
 
-    explicit constexpr operator int64_t() const
+    template<class Integral,
+             typename std::enable_if<(std::is_integral<Integral>::value
+                                      && sizeof(Integral) <= sizeof(UIntX2))
+                                    >::type* = nullptr>
+    explicit constexpr operator Integral() const
     {
-        return d.v01;
-    }
-
-    explicit constexpr operator uint32_t() const
-    {
-        return w.v0;
-    }
-
-    explicit constexpr operator int() const
-    {
-        return w.v0;
-    }
-
-    explicit constexpr operator uint16_t() const
-    {
-        return w.v0;
-    }
-
-    explicit constexpr operator uint8_t() const
-    {
-        return w.v0;
-    }
-
-    typedef typename std::conditional<std::is_same<uint64_t,
-                                                   unsigned long>::value,
-                                      unsigned long long,
-                                      unsigned long>::type
-            uint_missing_t;
-
-    explicit constexpr operator uint_missing_t() const
-    {
-        return d.v01;
+        return Integral(d.v01);
     }
 
     explicit constexpr operator bool() const
@@ -367,6 +393,9 @@ public:
     template<typename U, typename V>
     friend uint_x4<U,V> operator*(const uint_x4<U,V>&, const uint_x4<U,V>&);
 
+    template<typename U, typename V>
+    friend uint_x4<U,V> operator*(const uint_x4<U,V>&, V);
+
     template<typename U, typename V>
     friend std::pair< uint_x4<U,V>,uint_x4<U,V> >
         divmod(const uint_x4<U,V>&, const uint_x4<U,V>&);
@@ -378,10 +407,18 @@ public:
     friend uint_x4<U,V> operator-(const uint_x4<U,V>&, const uint_x4<U,V>&);
 
     template<typename U, typename V>
-    friend uint_x4<U,V> operator<<(const uint_x4<U,V>&, const uint_x4<U,V>&);
+    friend uint_x4<U,V> operator<<(const uint_x4<U,V>&, const bitcount_t shift);
 
     template<typename U, typename V>
-    friend uint_x4<U,V> operator>>(const uint_x4<U,V>&, const uint_x4<U,V>&);
+    friend uint_x4<U,V> operator>>(const uint_x4<U,V>&, const bitcount_t shift);
+
+#if PCG_64BIT_SPECIALIZATIONS
+    template<typename U>
+    friend uint_x4<U,uint64_t> operator<<(const uint_x4<U,uint64_t>&, const bitcount_t shift);
+
+    template<typename U>
+    friend uint_x4<U,uint64_t> operator>>(const uint_x4<U,uint64_t>&, const bitcount_t shift);
+#endif
 
     template<typename U, typename V>
     friend uint_x4<U,V> operator&(const uint_x4<U,V>&, const uint_x4<U,V>&);
@@ -422,12 +459,26 @@ public:
     template<typename U, typename V>
     friend bitcount_t trailingzeros(const uint_x4<U,V>&);
 
+#if PCG_64BIT_SPECIALIZATIONS
+    template<typename U>
+    friend bitcount_t flog2(const uint_x4<U,uint64_t>&);
+
+    template<typename U>
+    friend bitcount_t trailingzeros(const uint_x4<U,uint64_t>&);
+#endif
+
     uint_x4& operator*=(const uint_x4& rhs)
     {
         uint_x4 result = *this * rhs;
         return *this = result;
     }
 
+    uint_x4& operator*=(UIntX2 rhs)
+    {
+        uint_x4 result = *this * rhs;
+        return *this = result;
+    }
+
     uint_x4& operator/=(const uint_x4& rhs)
     {
         uint_x4 result = *this / rhs;
@@ -495,7 +546,7 @@ bitcount_t flog2(const uint_x4<U,V>& v)
 #endif
         if (v.wa[i] == 0)
              continue;
-        return flog2(v.wa[i]) + (sizeof(U)*CHAR_BIT)*i;
+        return flog2(v.wa[i]) + uint_x4<U,V>::UINT_BITS*i;
     }
     abort();
 }
@@ -510,11 +561,27 @@ bitcount_t trailingzeros(const uint_x4<U,V>& v)
         --i;
 #endif
         if (v.wa[i] != 0)
-            return trailingzeros(v.wa[i]) + (sizeof(U)*CHAR_BIT)*i;
+            return trailingzeros(v.wa[i]) + uint_x4<U,V>::UINT_BITS*i;
     }
-    return (sizeof(U)*CHAR_BIT)*4;
+    return uint_x4<U,V>::UINT_BITS*4;
 }
 
+#if PCG_64BIT_SPECIALIZATIONS
+template<typename UInt32>
+bitcount_t flog2(const uint_x4<UInt32,uint64_t>& v)
+{
+    return v.d.v23 > 0 ? flog2(v.d.v23) + uint_x4<UInt32,uint64_t>::UINT_BITS*2
+                       : flog2(v.d.v01);
+}
+
+template<typename UInt32>
+bitcount_t trailingzeros(const uint_x4<UInt32,uint64_t>& v)
+{
+    return v.d.v01 == 0 ? trailingzeros(v.d.v23) + uint_x4<UInt32,uint64_t>::UINT_BITS*2
+                        : trailingzeros(v.d.v01);
+}
+#endif
+
 template <typename UInt, typename UIntX2>
 std::pair< uint_x4<UInt,UIntX2>, uint_x4<UInt,UIntX2> >
     divmod(const uint_x4<UInt,UIntX2>& orig_dividend,
@@ -525,7 +592,7 @@ std::pair< uint_x4<UInt,UIntX2>, uint_x4<UInt,UIntX2> >
     // problematic because we can't take the log of zero.  (The boundary case
     // of division by zero is undefined.)
     if (orig_dividend < divisor)
-        return { uint_x4<UInt,UIntX2>(0UL), orig_dividend };
+        return { uint_x4<UInt,UIntX2>(UIntX2(0)), orig_dividend };
 
     auto dividend = orig_dividend;
 
@@ -534,7 +601,7 @@ std::pair< uint_x4<UInt,UIntX2>, uint_x4<UInt,UIntX2> >
     // assert(log2_dividend >= log2_divisor);
     bitcount_t logdiff = log2_dividend - log2_divisor;
 
-    constexpr uint_x4<UInt,UIntX2> ONE(1UL);
+    constexpr uint_x4<UInt,UIntX2> ONE(UIntX2(1));
     if (logdiff == 0)
         return { ONE, dividend - divisor };
 
@@ -543,7 +610,7 @@ std::pair< uint_x4<UInt,UIntX2>, uint_x4<UInt,UIntX2> >
     // to ensure that we *underestimate* the result.
     logdiff -= 1;
 
-    uint_x4<UInt,UIntX2> quotient(0UL);
+    uint_x4<UInt,UIntX2> quotient(UIntX2(0));
 
     auto qfactor = ONE << logdiff;
     auto factor  = divisor << logdiff;
@@ -579,15 +646,16 @@ template <typename UInt, typename UIntX2>
 uint_x4<UInt,UIntX2> operator*(const uint_x4<UInt,UIntX2>& a,
                                const uint_x4<UInt,UIntX2>& b)
 {
+    constexpr auto UINT_BITS = uint_x4<UInt,UIntX2>::UINT_BITS;
     uint_x4<UInt,UIntX2> r = {0U, 0U, 0U, 0U};
     bool carryin = false;
     bool carryout;
     UIntX2 a0b0 = UIntX2(a.w.v0) * UIntX2(b.w.v0);
     r.w.v0 = UInt(a0b0);
-    r.w.v1 = UInt(a0b0 >> 32);
+    r.w.v1 = UInt(a0b0 >> UINT_BITS);
 
     UIntX2 a1b0 = UIntX2(a.w.v1) * UIntX2(b.w.v0);
-    r.w.v2 = UInt(a1b0 >> 32);
+    r.w.v2 = UInt(a1b0 >> UINT_BITS);
     r.w.v1 = addwithcarry(r.w.v1, UInt(a1b0), carryin, &carryout);
     carryin = carryout;
     r.w.v2 = addwithcarry(r.w.v2, UInt(0U), carryin, &carryout);
@@ -596,7 +664,7 @@ uint_x4<UInt,UIntX2> operator*(const uint_x4<UInt,UIntX2>& a,
 
     UIntX2 a0b1 = UIntX2(a.w.v0) * UIntX2(b.w.v1);
     carryin = false;
-    r.w.v2 = addwithcarry(r.w.v2, UInt(a0b1 >> 32), carryin, &carryout);
+    r.w.v2 = addwithcarry(r.w.v2, UInt(a0b1 >> UINT_BITS), carryin, &carryout);
     carryin = carryout;
     r.w.v3 = addwithcarry(r.w.v3, UInt(0U), carryin, &carryout);
 
@@ -611,13 +679,82 @@ uint_x4<UInt,UIntX2> operator*(const uint_x4<UInt,UIntX2>& a,
     carryin = false;
     r.w.v2 = addwithcarry(r.w.v2, UInt(a1b1), carryin, &carryout);
     carryin = carryout;
-    r.w.v3 = addwithcarry(r.w.v3, UInt(a1b1 >> 32), carryin, &carryout);
+    r.w.v3 = addwithcarry(r.w.v3, UInt(a1b1 >> UINT_BITS), carryin, &carryout);
 
     r.d.v23 += a.d.v01 * b.d.v23 + a.d.v23 * b.d.v01;
 
     return r;
 }
 
+ 
+template <typename UInt, typename UIntX2>
+uint_x4<UInt,UIntX2> operator*(const uint_x4<UInt,UIntX2>& a,
+                               UIntX2 b01)
+{
+    constexpr auto UINT_BITS = uint_x4<UInt,UIntX2>::UINT_BITS;
+    uint_x4<UInt,UIntX2> r = {0U, 0U, 0U, 0U};
+    bool carryin = false;
+    bool carryout;
+    UIntX2 a0b0 = UIntX2(a.w.v0) * UIntX2(UInt(b01));
+    r.w.v0 = UInt(a0b0);
+    r.w.v1 = UInt(a0b0 >> UINT_BITS);
+
+    UIntX2 a1b0 = UIntX2(a.w.v1) * UIntX2(UInt(b01));
+    r.w.v2 = UInt(a1b0 >> UINT_BITS);
+    r.w.v1 = addwithcarry(r.w.v1, UInt(a1b0), carryin, &carryout);
+    carryin = carryout;
+    r.w.v2 = addwithcarry(r.w.v2, UInt(0U), carryin, &carryout);
+    carryin = carryout;
+    r.w.v3 = addwithcarry(r.w.v3, UInt(0U), carryin, &carryout);
+
+    UIntX2 a0b1 = UIntX2(a.w.v0) * UIntX2(b01 >> UINT_BITS);
+    carryin = false;
+    r.w.v2 = addwithcarry(r.w.v2, UInt(a0b1 >> UINT_BITS), carryin, &carryout);
+    carryin = carryout;
+    r.w.v3 = addwithcarry(r.w.v3, UInt(0U), carryin, &carryout);
+
+    carryin = false;
+    r.w.v1 = addwithcarry(r.w.v1, UInt(a0b1), carryin, &carryout);
+    carryin = carryout;
+    r.w.v2 = addwithcarry(r.w.v2, UInt(0U), carryin, &carryout);
+    carryin = carryout;
+    r.w.v3 = addwithcarry(r.w.v3, UInt(0U), carryin, &carryout);
+
+    UIntX2 a1b1 = UIntX2(a.w.v1) * UIntX2(b01 >> UINT_BITS);
+    carryin = false;
+    r.w.v2 = addwithcarry(r.w.v2, UInt(a1b1), carryin, &carryout);
+    carryin = carryout;
+    r.w.v3 = addwithcarry(r.w.v3, UInt(a1b1 >> UINT_BITS), carryin, &carryout);
+
+    r.d.v23 += a.d.v23 * b01;
+
+    return r;
+}
+
+#if PCG_64BIT_SPECIALIZATIONS
+#if defined(_MSC_VER)
+#pragma intrinsic(_umul128)
+#endif
+
+#if defined(_MSC_VER) || __SIZEOF_INT128__
+template <typename UInt32>
+uint_x4<UInt32,uint64_t> operator*(const uint_x4<UInt32,uint64_t>& a,
+				   const uint_x4<UInt32,uint64_t>& b)
+{
+#if defined(_MSC_VER)
+    uint64_t hi;
+    uint64_t lo = _umul128(a.d.v01, b.d.v01, &hi);
+#else
+    __uint128_t r = __uint128_t(a.d.v01) * __uint128_t(b.d.v01);
+    uint64_t lo = uint64_t(r);
+    uint64_t hi = r >> 64;
+#endif
+    hi += a.d.v23 * b.d.v01 + a.d.v01 * b.d.v23;
+    return {hi, lo};
+}
+#endif
+#endif
+
 
 template <typename UInt, typename UIntX2>
 uint_x4<UInt,UIntX2> operator+(const uint_x4<UInt,UIntX2>& a,
@@ -657,6 +794,37 @@ uint_x4<UInt,UIntX2> operator-(const uint_x4<UInt,UIntX2>& a,
     return r;
 }
 
+#if PCG_64BIT_SPECIALIZATIONS
+template <typename UInt32>
+uint_x4<UInt32,uint64_t> operator+(const uint_x4<UInt32,uint64_t>& a,
+				   const uint_x4<UInt32,uint64_t>& b)
+{
+    uint_x4<UInt32,uint64_t> r = {uint64_t(0u), uint64_t(0u)};
+
+    bool carryin = false;
+    bool carryout;
+    r.d.v01 = addwithcarry(a.d.v01, b.d.v01, carryin, &carryout);
+    carryin = carryout;
+    r.d.v23 = addwithcarry(a.d.v23, b.d.v23, carryin, &carryout);
+
+    return r;
+}
+
+template <typename UInt32>
+uint_x4<UInt32,uint64_t> operator-(const uint_x4<UInt32,uint64_t>& a,
+				   const uint_x4<UInt32,uint64_t>& b)
+{
+    uint_x4<UInt32,uint64_t> r = {uint64_t(0u), uint64_t(0u)};
+
+    bool carryin = false;
+    bool carryout;
+    r.d.v01 = subwithcarry(a.d.v01, b.d.v01, carryin, &carryout);
+    carryin = carryout;
+    r.d.v23 = subwithcarry(a.d.v23, b.d.v23, carryin, &carryout);
+
+    return r;
+}
+#endif
 
 template <typename UInt, typename UIntX2>
 uint_x4<UInt,UIntX2> operator&(const uint_x4<UInt,UIntX2>& a,
@@ -736,7 +904,7 @@ uint_x4<UInt,UIntX2> operator<<(const uint_x4<UInt,UIntX2>& v,
                                 const bitcount_t shift)
 {
     uint_x4<UInt,UIntX2> r = {0U, 0U, 0U, 0U};
-    const bitcount_t bits    = sizeof(UInt) * CHAR_BIT;
+    const bitcount_t bits    = uint_x4<UInt,UIntX2>::UINT_BITS;
     const bitcount_t bitmask = bits - 1;
     const bitcount_t shiftdiv = shift / bits;
     const bitcount_t shiftmod = shift & bitmask;
@@ -771,7 +939,7 @@ uint_x4<UInt,UIntX2> operator>>(const uint_x4<UInt,UIntX2>& v,
                                 const bitcount_t shift)
 {
     uint_x4<UInt,UIntX2> r = {0U, 0U, 0U, 0U};
-    const bitcount_t bits    = sizeof(UInt) * CHAR_BIT;
+    const bitcount_t bits    = uint_x4<UInt,UIntX2>::UINT_BITS;
     const bitcount_t bitmask = bits - 1;
     const bitcount_t shiftdiv = shift / bits;
     const bitcount_t shiftmod = shift & bitmask;
@@ -801,6 +969,38 @@ uint_x4<UInt,UIntX2> operator>>(const uint_x4<UInt,UIntX2>& v,
     return r;
 }
 
+#if PCG_64BIT_SPECIALIZATIONS
+template <typename UInt32>
+uint_x4<UInt32,uint64_t> operator<<(const uint_x4<UInt32,uint64_t>& v,
+				    const bitcount_t shift)
+{
+    constexpr bitcount_t bits2   = uint_x4<UInt32,uint64_t>::UINT_BITS * 2;
+    
+    if (shift >= bits2) {
+        return {v.d.v01 << (shift-bits2), uint64_t(0u)};
+    } else {
+        return {shift ? (v.d.v23 << shift) | (v.d.v01 >> (bits2-shift)) 
+                      : v.d.v23,
+                v.d.v01 << shift};
+    }
+}
+
+template <typename UInt32>
+uint_x4<UInt32,uint64_t> operator>>(const uint_x4<UInt32,uint64_t>& v,
+				    const bitcount_t shift)
+{
+    constexpr bitcount_t bits2   = uint_x4<UInt32,uint64_t>::UINT_BITS * 2;
+    
+    if (shift >= bits2) {
+        return {uint64_t(0u), v.d.v23 >> (shift-bits2)};
+    } else {
+        return {v.d.v23 >> shift,
+                shift ? (v.d.v01 >> shift) | (v.d.v23 << (bits2-shift))
+                      : v.d.v01};
+    }
+}
+#endif
+
 } // namespace pcg_extras
 
 #endif // PCG_UINT128_HPP_INCLUDED
